@@ -1,7 +1,7 @@
 /**
  * Mandelbulber v2, a 3D fractal generator       ,=#MKNmMMKmmßMNWy,
  *                                             ,B" ]L,,p%%%,,,§;, "K
- * Copyright (C) 2014-17 Mandelbulber Team     §R-==%w["'~5]m%=L.=~5N
+ * Copyright (C) 2014-23 Mandelbulber Team     §R-==%w["'~5]m%=L.=~5N
  *                                        ,=mm=§M ]=4 yJKA"/-Nsaj  "Bw,==,,
  * This file is part of Mandelbulber.    §R.r= jw",M  Km .mM  FW ",§=ß., ,TN
  *                                     ,4R =%["w[N=7]J '"5=],""]]M,w,-; T=]M
@@ -27,7 +27,7 @@
  *
  * ###########################################################################
  *
- * Authors: Krzysztof Marczak (buddhi1980@gmail.com)
+ * Authors: Krzysztof Marczak (buddhi1980@gmail.com), Robert Pancoast (RobertPancoast77@gmail.com)
  *
  * CVector2, CVector3, CVector4 and CMatrix33 classes - vector and matrix
  * manipulation
@@ -44,21 +44,13 @@
 #define _USE_MATH_DEFINES
 #endif
 
-#include <QString>
 #include <gsl/gsl_sys.h>
-
-#if defined(_MSC_VER)
-#pragma warning(push)
-#pragma warning(disable : 4005) // macro redefinition
-#endif
-
-// MSVC defines math constants elsewhere, do not warn about the
-// redefinition in math.h
 #include <math.h>
 
-#if defined(_MSC_VER)
-#pragma warning(pop)
-#endif
+#include <QString>
+#include <array>
+
+// #include "include_header_wrapper.hpp"
 
 /************************* vector 3D **********************/
 class CVector3
@@ -88,11 +80,18 @@ public:
 		y = vector.y;
 		z = vector.z;
 	}
-	inline CVector3(const double vector[3])
+	inline explicit CVector3(const double vector[3])
 	{
-		x = vector[0];
-		y = vector[1];
-		z = vector[2];
+		if (vector)
+		{
+			x = vector[0];
+			y = vector[1];
+			z = vector[2];
+		}
+		else
+		{
+			qFatal("Assigned nullptr to CVector3 as initialization!");
+		}
 	}
 	inline CVector3 operator+(const CVector3 &vector) const
 	{
@@ -107,7 +106,7 @@ public:
 		return CVector3((vector.x > 0.0 ? fmod(x, vector.x) : x),
 			(vector.y > 0.0 ? fmod(y, vector.y) : y), (vector.z > 0.0 ? fmod(z, vector.z) : z));
 	}
-	inline CVector3 mod(const CVector3 &vector) const
+	inline CVector3 repeatMod(const CVector3 &vector) const
 	{
 		if (vector.Length() == 0.0) return *this;
 		return (((*this - vector * 0.5) % vector) + vector) % vector - vector * 0.5;
@@ -124,13 +123,11 @@ public:
 	{
 		return CVector3(x / scalar, y / scalar, z / scalar);
 	}
-	inline CVector3 &operator=(const CVector3 &vector)
+	inline CVector3 operator/(const CVector3 &vector) const
 	{
-		x = vector.x;
-		y = vector.y;
-		z = vector.z;
-		return *this;
+		return CVector3(x / vector.x, y / vector.y, z / vector.z);
 	}
+	inline CVector3 &operator=(const CVector3 &vector) = default;
 	inline CVector3 &operator+=(const CVector3 &vector)
 	{
 		x += vector.x;
@@ -178,6 +175,10 @@ public:
 		return x == vector.x && y == vector.y && z == vector.z;
 	}
 	inline double Length() const { return sqrt(x * x + y * y + z * z); }
+	inline double LengthPow(double p) const
+	{
+		return pow(pow(x, p) + pow(y, p) + pow(z, p), 1.0 / p);
+	}
 	inline double Dot(const CVector3 &vector) const
 	{
 		return x * vector.x + y * vector.y + z * vector.z;
@@ -200,7 +201,7 @@ public:
 	}
 	inline double Normalize() // returns normalization factor
 	{
-		double norm = 1.0 / Length();
+		const double norm = 1.0 / Length();
 		x = x * norm;
 		y = y * norm;
 		z = z * norm;
@@ -216,7 +217,7 @@ public:
 		// than evaluating each dimension sequentially.
 		return !(gsl_finite(x) & gsl_finite(y) & gsl_finite(z));
 	}
-	CVector3 RotateAroundVectorByAngle(CVector3 axis, double angle) const;
+	CVector3 RotateAroundVectorByAngle(const CVector3 &axis, double angle) const;
 	QString Debug() const
 	{
 		return QString("[%1, %2, %3]").arg(QString::number(x), QString::number(y), QString::number(z));
@@ -229,12 +230,12 @@ public:
 	double z;
 };
 
-inline CVector3 operator*(double scalar, CVector3 vector)
+inline CVector3 operator*(double scalar, const CVector3 &vector)
 {
 	return CVector3(vector.x * scalar, vector.y * scalar, vector.z * scalar);
 }
 
-inline CVector3 operator/(double scalar, CVector3 vector)
+inline CVector3 operator/(double scalar, const CVector3 &vector)
 {
 	return CVector3(scalar / vector.x, scalar / vector.y, scalar / vector.z);
 }
@@ -246,6 +247,16 @@ inline CVector3 fabs(CVector3 v)
 	return v;
 }
 
+inline CVector3 max(const CVector3 &v1, const CVector3 &v2)
+{
+	return CVector3(std::max(v1.x, v2.x), std::max(v1.y, v2.y), std::max(v1.z, v2.z));
+}
+
+inline CVector3 min(const CVector3 &v1, const CVector3 &v2)
+{
+	return CVector3(std::min(v1.x, v2.x), std::min(v1.y, v2.y), std::min(v1.z, v2.z));
+}
+
 /******************* Structured vector 3D ****************/
 class StructuredCVector3
 {
@@ -260,7 +271,7 @@ public:
 		}
 	}
 
-	inline StructuredCVector3(CVector3 vectors[])
+	inline explicit StructuredCVector3(CVector3 vectors[])
 	{
 		for (int n = 0; n < 8; n++)
 		{
@@ -270,9 +281,9 @@ public:
 		}
 	}
 
-	double x[8];
-	double y[8];
-	double z[8];
+	double x[8]{};
+	double y[8]{};
+	double z[8]{};
 };
 
 /************************* vector 4D **********************/
@@ -293,7 +304,7 @@ public:
 		z = _z;
 		w = _w;
 	}
-	inline CVector4(CVector3 vectXYZ, double _w)
+	inline CVector4(const CVector3 &vectXYZ, double _w)
 	{
 		x = vectXYZ.x;
 		y = vectXYZ.y;
@@ -307,7 +318,7 @@ public:
 		z = v.z;
 		w = v.w;
 	}
-	inline CVector4(const double v[4])
+	inline explicit CVector4(const double v[4])
 	{
 		x = v[0];
 		y = v[1];
@@ -327,15 +338,13 @@ public:
 	{
 		return CVector4(x * v.x, y * v.y, z * v.z, w * v.w);
 	}
-	inline CVector4 operator/(const double &s) const { return CVector4(x / s, y / s, z / s, w / s); }
-	inline CVector4 &operator=(const CVector4 &v)
+	inline CVector4 operator*(const CVector3 &v) const
 	{
-		x = v.x;
-		y = v.y;
-		z = v.z;
-		w = v.w;
-		return *this;
+		return CVector4(x * v.x, y * v.y, z * v.z, w);
 	}
+
+	inline CVector4 operator/(const double &s) const { return CVector4(x / s, y / s, z / s, w / s); }
+	inline CVector4 &operator=(const CVector4 &v) = default;
 	inline CVector4 &operator+=(const CVector4 &v)
 	{
 		x += v.x;
@@ -368,6 +377,14 @@ public:
 		w *= v.w;
 		return *this;
 	}
+	inline CVector4 &operator*=(const CVector3 &v)
+	{
+		x *= v.x;
+		y *= v.y;
+		z *= v.z;
+		w *= 1.0;
+		return *this;
+	}
 	inline CVector4 &operator/=(const double &s)
 	{
 		x /= s;
@@ -376,6 +393,7 @@ public:
 		w /= s;
 		return *this;
 	}
+
 	inline bool operator==(const CVector4 &v) const
 	{
 		return x == v.x && y == v.y && z == v.z && w == v.w;
@@ -391,18 +409,20 @@ public:
 		c.w = fabs(w);
 		return c;
 	}
-	inline void GetXYZWInto(CVector3 &v, double &w) const
+	inline void GetXYZWInto(CVector3 &v, double &wOutput) const
 	{
 		v.x = x;
 		v.y = y;
 		v.z = z;
-		w = this->w;
+		wOutput = w;
 	}
 	inline CVector3 GetXYZ() const { return CVector3(x, y, z); }
 
+	inline std::array<double, 4> GetArray() const { return {x, y, z, w}; }
+
 	inline double Normalize()
 	{
-		double norm = 1.0 / Length();
+		const double norm = 1.0 / Length();
 		x = x * norm;
 		y = y * norm;
 		z = z * norm;
@@ -420,6 +440,18 @@ public:
 		if (v.Length() == 0.0) return *this;
 		return (((*this - v * 0.5) % v) + v) % v - v * 0.5;
 	}
+
+	inline bool IsNotANumber() const
+	{
+		// Check for NaN
+		// single "&" is intended: since "most of times" gsl_finite is true,
+		// always evaluating xyz at once allows cpu to perform faster
+		// than evaluating each dimension sequentially.
+		return !(gsl_finite(x) & gsl_finite(y) & gsl_finite(z) & gsl_finite(w));
+	}
+
+	CVector4 RotateAroundVectorByAngle(const CVector3 &axis, double angle) const;
+
 	QString Debug() const
 	{
 		return QString("[%1, %2, %3, %4]")
@@ -428,15 +460,18 @@ public:
 
 	double itemByName(char item) const;
 
-	double x, y, z, w;
+	double x;
+	double y;
+	double z;
+	double w;
 };
 
-inline CVector4 operator*(double scalar, CVector4 vector)
+inline CVector4 operator*(double scalar, const CVector4 &vector)
 {
 	return CVector4(vector.x * scalar, vector.y * scalar, vector.z * scalar, vector.w * scalar);
 }
 
-inline CVector4 operator/(double scalar, CVector4 vector)
+inline CVector4 operator/(double scalar, const CVector4 &vector)
 {
 	return CVector4(scalar / vector.x, scalar / vector.y, scalar / vector.z, scalar / vector.w);
 }
@@ -503,6 +538,7 @@ public:
 		return *this;
 	}
 	inline double Length() const { return sqrt(x * x + y * y); }
+	inline double LengthPow(double p) const { return pow(pow(x, p) + pow(y, p), 1.0 / p); }
 	inline CVector2 Deadband() const
 	{
 		CVector2 c;
@@ -532,7 +568,7 @@ public:
 	}
 	inline double Normalize() // returns normalization factor
 	{
-		double norm = 1.0 / Length();
+		const double norm = 1.0 / Length();
 		x = x * norm;
 		y = y * norm;
 		return norm;
@@ -594,14 +630,17 @@ public:
 	void RotateZ(double angle);
 	void Null();
 	CVector3 RotateVector(const CVector3 &vector) const;
+	CVector4 RotateVector(const CVector4 &vector) const;
 	double GetAlfa() const;
 	double GetBeta() const;
 	double GetGamma() const;
 	void SetRotation(double angles[3]);
 	void SetRotation(double alpha, double beta, double gamma);
-	void SetRotation(CVector3 rotation);
-	void SetRotation2(CVector3 rotation);
-	void SetRotation3(CVector3 rotation);
+	void SetRotation(const CVector3 &rotation);
+	void SetRotation(const CVector3 &direction, const CVector3 &up);
+	void SetRotation2(const CVector3 &rotation);
+	void SetRotation3(const CVector3 &rotation);
+	void SetRotation4(const CVector3 &rotation);
 	CRotationMatrix Transpose() const;
 	CMatrix33 GetMatrix() const { return matrix; }
 
@@ -663,8 +702,8 @@ public:
 	// void SetRotation2(CVector4 rotation);
 	// void SetRotation3(CVector4 rotation);
 
-	void SetRotation44a(CVector3 rotation);
-	void SetRotation44b(CVector3 rotation);
+	void SetRotation44a(const CVector3 &rotation);
+	void SetRotation44b(const CVector3 &rotation);
 
 	CRotationMatrix44 Transpose() const;
 	CMatrix44 GetMatrix() const { return matrix; }
