@@ -1,7 +1,7 @@
 /**
  * Mandelbulber v2, a 3D fractal generator       ,=#MKNmMMKmmßMNWy,
  *                                             ,B" ]L,,p%%%,,,§;, "K
- * Copyright (C) 2016-17 Mandelbulber Team     §R-==%w["'~5]m%=L.=~5N
+ * Copyright (C) 2016-21 Mandelbulber Team     §R-==%w["'~5]m%=L.=~5N
  *                                        ,=mm=§M ]=4 yJKA"/-Nsaj  "Bw,==,,
  * This file is part of Mandelbulber.    §R.r= jw",M  Km .mM  FW ",§=ß., ,TN
  *                                     ,4R =%["w[N=7]J '"5=],""]]M,w,-; T=]M
@@ -41,399 +41,409 @@
 #include <QtWidgets/QtWidgets>
 
 #include "synchronize_interface.hpp"
-#include "system.hpp"
+#include "system_data.hpp"
 
-cAutomatedWidgets::cAutomatedWidgets(QObject *parent) : QObject(parent)
-{
-}
+cAutomatedWidgets::cAutomatedWidgets(QObject *parent) : QObject(parent) {}
 
 void cAutomatedWidgets::slotSliderMoved(int value) const
 {
 	using namespace std;
-	QString sliderName = this->sender()->objectName();
-	QString type, parameterName;
+	const QString sliderName = sender()->objectName();
+	QString type;
+	QString parameterName;
 	GetNameAndType(sliderName, &parameterName, &type);
-	QString spinBoxName = QString("spinbox_") + parameterName;
+	const QString spinBoxName = QString("spinbox_") + parameterName;
 
-	QDoubleSpinBox *spinBox = this->sender()->parent()->findChild<QDoubleSpinBox *>(spinBoxName);
+	QDoubleSpinBox *spinBox = sender()->parent()->findChild<QDoubleSpinBox *>(spinBoxName);
 	if (spinBox)
 	{
-		double decimals = spinBox->decimals();
-		double divider = pow(10.0, decimals);
+		const double decimals = spinBox->decimals();
+		const double divider = pow(10.0, decimals);
 		spinBox->setValue(value / divider);
 	}
 	else
 	{
-		qWarning() << "slotSliderMoved() error: spinbox " << spinBoxName << " doesn't exists" << endl;
+		qWarning() << "slotSliderMoved() error: spinbox " << spinBoxName << " doesn't exists";
 	}
 }
 
 void cAutomatedWidgets::slotDoubleSpinBoxChanged(double value) const
 {
 	using namespace std;
-	QString spinBoxName = this->sender()->objectName();
-	QString type, parameterName;
+	const QString spinBoxName = sender()->objectName();
+	QString type;
+	QString parameterName;
 	GetNameAndType(spinBoxName, &parameterName, &type);
-	QString sliderName = QString("slider_") + parameterName;
-	QSlider *slider = this->sender()->parent()->findChild<QSlider *>(sliderName);
+	const QString sliderName = QString("slider_") + parameterName;
+	QSlider *slider = sender()->parent()->findChild<QSlider *>(sliderName);
 	if (slider)
 	{
-		QDoubleSpinBox *spinBox = static_cast<QDoubleSpinBox *>(this->sender());
-		double decimals = spinBox->decimals();
-		double multiplier = pow(10.0, decimals);
+		QDoubleSpinBox *spinBox = static_cast<QDoubleSpinBox *>(sender());
+		const double decimals = spinBox->decimals();
+		const double multiplier = pow(10.0, decimals);
 		slider->setValue(int(value * multiplier));
 	}
 	else
 	{
-		qWarning() << "slotDoubleSpinBoxChanged() error: slider " << sliderName << " doesn't exists"
-							 << endl;
+		qWarning() << "slotDoubleSpinBoxChanged() error: slider " << sliderName << " doesn't exists";
 	}
 }
 
 void cAutomatedWidgets::slotIntSliderMoved(int value) const
 {
 	using namespace std;
-	QString sliderName = this->sender()->objectName();
-	QString type, parameterName;
+	const QString sliderName = sender()->objectName();
+	QString type;
+	QString parameterName;
 	GetNameAndType(sliderName, &parameterName, &type);
-	QString spinboxName = QString("spinboxInt_") + parameterName;
+	const QString spinboxName = QString("spinboxInt_") + parameterName;
 
-	QSpinBox *spinbox = this->sender()->parent()->findChild<QSpinBox *>(spinboxName);
+	QSpinBox *spinbox = sender()->parent()->findChild<QSpinBox *>(spinboxName);
 	if (spinbox)
 	{
 		spinbox->setValue(value);
 	}
 	else
 	{
-		qWarning() << "slotLogSliderMoved() error: lineEdit " << spinboxName << " doesn't exists"
-							 << endl;
+		qWarning() << "slotLogSliderMoved() error: lineEdit " << spinboxName << " doesn't exists";
 	}
 }
 
 void cAutomatedWidgets::slotIntSpinBoxChanged(int value) const
 {
 	using namespace std;
-	QString spinBoxName = this->sender()->objectName();
-	QString type, parameterName;
+	const QString spinBoxName = sender()->objectName();
+	QString type;
+	QString parameterName;
 	GetNameAndType(spinBoxName, &parameterName, &type);
-	QString sliderName = QString("sliderInt_") + parameterName;
+	const QString sliderName = QString("sliderInt_") + parameterName;
 
-	QSlider *slider = this->sender()->parent()->findChild<QSlider *>(sliderName);
+	QSlider *slider = sender()->parent()->findChild<QSlider *>(sliderName);
 	if (slider)
 	{
 		slider->setValue(value);
 	}
 	else
 	{
-		qWarning() << "slotIntSpinBoxChanged() error: slider " << sliderName << " doesn't exists"
-							 << endl;
+		qWarning() << "slotIntSpinBoxChanged() error: slider " << sliderName << " doesn't exists";
 	}
 }
 
 void cAutomatedWidgets::slotSlider3Moved(int value) const
 {
 	using namespace std;
-	QString sliderName = this->sender()->objectName();
-	QString type, parameterName;
+	const QString sliderName = sender()->objectName();
+	QString type;
+	QString parameterName;
 	GetNameAndType(sliderName, &parameterName, &type);
-	QString spinBoxName = QString("spinbox3_") + parameterName;
+	const QString spinBoxName = QString("spinbox3_") + parameterName;
 
-	QDoubleSpinBox *spinBox = this->sender()->parent()->findChild<QDoubleSpinBox *>(spinBoxName);
+	QDoubleSpinBox *spinBox = sender()->parent()->findChild<QDoubleSpinBox *>(spinBoxName);
 
 	if (spinBox)
 	{
-		double decimals = spinBox->decimals();
-		double divider = pow(10.0, decimals);
+		const double decimals = spinBox->decimals();
+		const double divider = pow(10.0, decimals);
 		spinBox->setValue(value / divider);
 	}
 	else
 	{
-		qWarning() << "slotSlider3Moved() error: spinbox " << spinBoxName << " doesn't exists" << endl;
+		qWarning() << "slotSlider3Moved() error: spinbox " << spinBoxName << " doesn't exists";
 	}
 }
 
 void cAutomatedWidgets::slotSlider4Moved(int value) const
 {
 	using namespace std;
-	QString sliderName = this->sender()->objectName();
-	QString type, parameterName;
+	const QString sliderName = sender()->objectName();
+	QString type;
+	QString parameterName;
 	GetNameAndType(sliderName, &parameterName, &type);
-	QString spinBoxName = QString("spinbox4_") + parameterName;
+	const QString spinBoxName = QString("spinbox4_") + parameterName;
 
-	QDoubleSpinBox *spinBox = this->sender()->parent()->findChild<QDoubleSpinBox *>(spinBoxName);
+	QDoubleSpinBox *spinBox = sender()->parent()->findChild<QDoubleSpinBox *>(spinBoxName);
 
 	if (spinBox)
 	{
-		double decimals = spinBox->decimals();
-		double divider = pow(10.0, decimals);
+		const double decimals = spinBox->decimals();
+		const double divider = pow(10.0, decimals);
 		spinBox->setValue(value / divider);
 	}
 	else
 	{
-		qWarning() << "slotSlider4Moved() error: spinbox " << spinBoxName << " doesn't exists" << endl;
+		qWarning() << "slotSlider4Moved() error: spinbox " << spinBoxName << " doesn't exists";
 	}
 }
 
 void cAutomatedWidgets::slotSpinBox3Changed(double value) const
 {
 	using namespace std;
-	QString spinBoxName = this->sender()->objectName();
-	QString type, parameterName;
+	const QString spinBoxName = sender()->objectName();
+	QString type;
+	QString parameterName;
 	GetNameAndType(spinBoxName, &parameterName, &type);
-	QString sliderName = QString("slider3_") + parameterName;
+	const QString sliderName = QString("slider3_") + parameterName;
 
-	QSlider *slider = this->sender()->parent()->findChild<QSlider *>(sliderName);
+	QSlider *slider = sender()->parent()->findChild<QSlider *>(sliderName);
 
 	if (slider)
 	{
-		QDoubleSpinBox *spinBox = static_cast<QDoubleSpinBox *>(this->sender());
-		double decimals = spinBox->decimals();
-		double multiplier = pow(10.0, decimals);
+		QDoubleSpinBox *spinBox = static_cast<QDoubleSpinBox *>(sender());
+		const double decimals = spinBox->decimals();
+		const double multiplier = pow(10.0, decimals);
 		slider->setValue(int(value * multiplier));
 	}
 	else
 	{
-		qWarning() << "slotSpinBox3Changed() error: slider " << sliderName << " doesn't exists" << endl;
+		qWarning() << "slotSpinBox3Changed() error: slider " << sliderName << " doesn't exists";
 	}
 }
 
 void cAutomatedWidgets::slotSpinBox4Changed(double value) const
 {
 	using namespace std;
-	QString spinBoxName = this->sender()->objectName();
-	QString type, parameterName;
+	const QString spinBoxName = sender()->objectName();
+	QString type;
+	QString parameterName;
 	GetNameAndType(spinBoxName, &parameterName, &type);
-	QString sliderName = QString("slider4_") + parameterName;
+	const QString sliderName = QString("slider4_") + parameterName;
 
-	QSlider *slider = this->sender()->parent()->findChild<QSlider *>(sliderName);
+	QSlider *slider = sender()->parent()->findChild<QSlider *>(sliderName);
 
 	if (slider)
 	{
-		QDoubleSpinBox *spinBox = static_cast<QDoubleSpinBox *>(this->sender());
-		double decimals = spinBox->decimals();
-		double multiplier = pow(10.0, decimals);
+		QDoubleSpinBox *spinBox = static_cast<QDoubleSpinBox *>(sender());
+		const double decimals = spinBox->decimals();
+		const double multiplier = pow(10.0, decimals);
 		slider->setValue(int(value * multiplier));
 	}
 	else
 	{
-		qWarning() << "slotSpinBox4Changed() error: slider " << sliderName << " doesn't exists" << endl;
+		qWarning() << "slotSpinBox4Changed() error: slider " << sliderName << " doesn't exists";
 	}
 }
 
 void cAutomatedWidgets::slotDial3Moved(int value) const
 {
 	using namespace std;
-	QString sliderName = this->sender()->objectName();
-	QString type, parameterName;
+	const QString sliderName = sender()->objectName();
+	QString type;
+	QString parameterName;
 	GetNameAndType(sliderName, &parameterName, &type);
-	QString spinBoxName = QString("spinboxd3_") + parameterName;
+	const QString spinBoxName = QString("spinboxd3_") + parameterName;
 
-	QDoubleSpinBox *spinBox = this->sender()->parent()->findChild<QDoubleSpinBox *>(spinBoxName);
+	QDoubleSpinBox *spinBox = sender()->parent()->findChild<QDoubleSpinBox *>(spinBoxName);
 	if (spinBox)
 	{
 		spinBox->setValue(value / 100.0);
 	}
 	else
 	{
-		qWarning() << "slotDial3Moved() error: spinbox " << spinBoxName << " doesn't exists" << endl;
+		qWarning() << "slotDial3Moved() error: spinbox " << spinBoxName << " doesn't exists";
 	}
 }
 
 void cAutomatedWidgets::slotDial4Moved(int value) const
 {
 	using namespace std;
-	QString sliderName = this->sender()->objectName();
-	QString type, parameterName;
+	const QString sliderName = sender()->objectName();
+	QString type;
+	QString parameterName;
 	GetNameAndType(sliderName, &parameterName, &type);
-	QString spinBoxName = QString("spinboxd4_") + parameterName;
+	const QString spinBoxName = QString("spinboxd4_") + parameterName;
 
-	QDoubleSpinBox *spinBox = this->sender()->parent()->findChild<QDoubleSpinBox *>(spinBoxName);
+	QDoubleSpinBox *spinBox = sender()->parent()->findChild<QDoubleSpinBox *>(spinBoxName);
 	if (spinBox)
 	{
 		spinBox->setValue(value / 100.0);
 	}
 	else
 	{
-		qWarning() << "slotDial4Moved() error: spinbox " << spinBoxName << " doesn't exists" << endl;
+		qWarning() << "slotDial4Moved() error: spinbox " << spinBoxName << " doesn't exists";
 	}
 }
 
 void cAutomatedWidgets::slotSpinBoxD3Changed(double value) const
 {
 	using namespace std;
-	QString spinBoxName = this->sender()->objectName();
-	QString type, parameterName;
+	const QString spinBoxName = sender()->objectName();
+	QString type;
+	QString parameterName;
 	GetNameAndType(spinBoxName, &parameterName, &type);
-	QString dialName = QString("dial3_") + parameterName;
+	const QString dialName = QString("dial3_") + parameterName;
 
-	QDial *dial = this->sender()->parent()->findChild<QDial *>(dialName);
+	QDial *dial = sender()->parent()->findChild<QDial *>(dialName);
 	if (dial)
 	{
 		dial->setValue(int(value * 100.0));
 	}
 	else
 	{
-		qWarning() << "slotSpinBox3Changed() error: slider " << dialName << " doesn't exists" << endl;
+		qWarning() << "slotSpinBox3Changed() error: slider " << dialName << " doesn't exists";
 	}
 }
 
 void cAutomatedWidgets::slotSpinBoxD4Changed(double value) const
 {
 	using namespace std;
-	QString spinBoxName = this->sender()->objectName();
-	QString type, parameterName;
+	const QString spinBoxName = sender()->objectName();
+	QString type;
+	QString parameterName;
 	GetNameAndType(spinBoxName, &parameterName, &type);
-	QString dialName = QString("dial4_") + parameterName;
+	const QString dialName = QString("dial4_") + parameterName;
 
-	QDial *dial = this->sender()->parent()->findChild<QDial *>(dialName);
+	QDial *dial = sender()->parent()->findChild<QDial *>(dialName);
 	if (dial)
 	{
 		dial->setValue(int(value * 100.0));
 	}
 	else
 	{
-		qWarning() << "slotSpinBox4Changed() error: slider " << dialName << " doesn't exists" << endl;
+		qWarning() << "slotSpinBox4Changed() error: slider " << dialName << " doesn't exists";
 	}
 }
 
 void cAutomatedWidgets::slotDialMoved(int value) const
 {
 	using namespace std;
-	QString sliderName = this->sender()->objectName();
-	QString type, parameterName;
+	const QString sliderName = sender()->objectName();
+	QString type;
+	QString parameterName;
 	GetNameAndType(sliderName, &parameterName, &type);
-	QString spinBoxName = QString("spinboxd_") + parameterName;
+	const QString spinBoxName = QString("spinboxd_") + parameterName;
 
-	QDoubleSpinBox *spinBox = this->sender()->parent()->findChild<QDoubleSpinBox *>(spinBoxName);
+	QDoubleSpinBox *spinBox = sender()->parent()->findChild<QDoubleSpinBox *>(spinBoxName);
 	if (spinBox)
 	{
 		spinBox->setValue(value / 100.0);
 	}
 	else
 	{
-		qWarning() << "slotDialMoved() error: spinbox " << spinBoxName << " doesn't exists" << endl;
+		qWarning() << "slotDialMoved() error: spinbox " << spinBoxName << " doesn't exists";
 	}
 }
 
 void cAutomatedWidgets::slotSpinBoxDChanged(double value) const
 {
 	using namespace std;
-	QString spinBoxName = this->sender()->objectName();
-	QString type, parameterName;
+	const QString spinBoxName = sender()->objectName();
+	QString type;
+	QString parameterName;
 	GetNameAndType(spinBoxName, &parameterName, &type);
-	QString dialName = QString("dial_") + parameterName;
+	const QString dialName = QString("dial_") + parameterName;
 
-	QDial *dial = this->sender()->parent()->findChild<QDial *>(dialName);
+	QDial *dial = sender()->parent()->findChild<QDial *>(dialName);
 	if (dial)
 	{
 		dial->setValue(int(value * 100.0));
 	}
 	else
 	{
-		qWarning() << "slotSpinBoxChanged() error: slider " << dialName << " doesn't exists" << endl;
+		qWarning() << "slotSpinBoxChanged() error: slider " << dialName << " doesn't exists";
 	}
 }
 
 void cAutomatedWidgets::slotLogSliderMoved(int value) const
 {
 	using namespace std;
-	QString sliderName = this->sender()->objectName();
-	QString type, parameterName;
+	const QString sliderName = sender()->objectName();
+	QString type;
+	QString parameterName;
 	GetNameAndType(sliderName, &parameterName, &type);
-	QString lineEditName = QString("logedit_") + parameterName;
+	const QString lineEditName = QString("logedit_") + parameterName;
 
-	QLineEdit *lineEdit = this->sender()->parent()->findChild<QLineEdit *>(lineEditName);
+	QLineEdit *lineEdit = sender()->parent()->findChild<QLineEdit *>(lineEditName);
 	if (lineEdit)
 	{
-		double dValue = pow(10.0, value / 100.0);
-		QString text = QString("%L1").arg(dValue);
+		const double dValue = pow(10.0, value / 100.0);
+		const QString text = QString("%L1").arg(dValue);
 		lineEdit->setText(text);
 	}
 	else
 	{
-		qWarning() << "slotLogSliderMoved() error: lineEdit " << lineEditName << " doesn't exists"
-							 << endl;
+		qWarning() << "slotLogSliderMoved() error: lineEdit " << lineEditName << " doesn't exists";
 	}
 }
 
 void cAutomatedWidgets::slotLogSliderVect3Moved(int value) const
 {
 	using namespace std;
-	QString sliderName = this->sender()->objectName();
-	QString type, parameterName;
+	const QString sliderName = sender()->objectName();
+	QString type;
+	QString parameterName;
 	GetNameAndType(sliderName, &parameterName, &type);
-	QString lineEditName = QString("logvect3_") + parameterName;
+	const QString lineEditName = QString("logvect3_") + parameterName;
 
-	QLineEdit *lineEdit = this->sender()->parent()->findChild<QLineEdit *>(lineEditName);
+	QLineEdit *lineEdit = sender()->parent()->findChild<QLineEdit *>(lineEditName);
 	if (lineEdit)
 	{
-		double dValue = pow(10.0, value / 100.0);
-		QString text = QString("%L1").arg(dValue);
+		const double dValue = pow(10.0, value / 100.0);
+		const QString text = QString("%L1").arg(dValue);
 		lineEdit->setText(text);
 	}
 	else
 	{
-		qWarning() << "slotLogSliderVect3Moved() error: lineEdit " << lineEditName << " doesn't exists"
-							 << endl;
+		qWarning() << "slotLogSliderVect3Moved() error: lineEdit " << lineEditName << " doesn't exists";
 	}
 }
 
 void cAutomatedWidgets::slotLogLineEditChanged(const QString &text) const
 {
 	using namespace std;
-	QString lineEditName = this->sender()->objectName();
-	QString type, parameterName;
+	const QString lineEditName = sender()->objectName();
+	QString type;
+	QString parameterName;
 	GetNameAndType(lineEditName, &parameterName, &type);
-	QString sliderName = QString("logslider_") + parameterName;
+	const QString sliderName = QString("logslider_") + parameterName;
 
-	QSlider *slider = this->sender()->parent()->findChild<QSlider *>(sliderName);
+	QSlider *slider = sender()->parent()->findChild<QSlider *>(sliderName);
 	if (slider)
 	{
-		double value = systemData.locale.toDouble(text);
+		const double value = systemData.locale.toDouble(text);
 		if (value > 0.0)
 		{
-			int sliderPosition = int(log10(systemData.locale.toDouble(text)) * 100.0);
+			const int sliderPosition = int(log10(systemData.locale.toDouble(text)) * 100.0);
 			slider->setValue(sliderPosition);
 		}
 		else
 		{
 			qWarning() << "slotLogLineEditChanged() error: value from " << lineEditName
-								 << " is not greater zero" << endl;
+								 << " is not greater zero";
 		}
 	}
 	else
 	{
-		qWarning() << "slotLogLineEditChanged() error: slider " << sliderName << " doesn't exists"
-							 << endl;
+		qWarning() << "slotLogLineEditChanged() error: slider " << sliderName << " doesn't exists";
 	}
 }
 
 void cAutomatedWidgets::slotLogVect3Changed(const QString &text) const
 {
 	using namespace std;
-	QString lineEditName = this->sender()->objectName();
-	QString type, parameterName;
+	const QString lineEditName = sender()->objectName();
+	QString type;
+	QString parameterName;
 	GetNameAndType(lineEditName, &parameterName, &type);
-	QString sliderName = QString("logslidervect3_") + parameterName;
+	const QString sliderName = QString("logslidervect3_") + parameterName;
 
-	QSlider *slider = this->sender()->parent()->findChild<QSlider *>(sliderName);
+	QSlider *slider = sender()->parent()->findChild<QSlider *>(sliderName);
 	if (slider)
 	{
-		double value = systemData.locale.toDouble(text);
+		const double value = systemData.locale.toDouble(text);
 		if (value > 0.0)
 		{
-			int sliderPosition = int(log10(systemData.locale.toDouble(text)) * 100.0);
+			const int sliderPosition = int(log10(systemData.locale.toDouble(text)) * 100.0);
 			slider->setValue(sliderPosition);
 		}
 		else
 		{
 			qWarning() << "slotLogVect3Changed() error: value from " << lineEditName
-								 << " is not greater zero" << endl;
+								 << " is not greater zero";
 		}
 	}
 	else
 	{
-		qWarning() << "slotLogVect3Changed() error: slider " << sliderName << " doesn't exists" << endl;
+		qWarning() << "slotLogVect3Changed() error: slider " << sliderName << " doesn't exists";
 	}
 }
 
@@ -444,22 +454,22 @@ void cAutomatedWidgets::slotLogVect3Changed(const QString &text) const
 void cAutomatedWidgets::ConnectSignalsForSlidersInWindow(QWidget *window) const
 {
 	QList<QSlider *> widgetList = window->findChildren<QSlider *>();
-	QList<QSlider *>::iterator it;
-	for (it = widgetList.begin(); it != widgetList.end(); ++it)
+	for (auto &it : widgetList)
 	{
-		QString name = (*it)->objectName();
-		if (name.length() > 1 && (*it)->metaObject()->className() == QString("QSlider"))
+		QString name = it->objectName();
+		if (name.length() > 1 && it->metaObject()->className() == QString("QSlider"))
 		{
-			const QSlider *slider = *it;
+			const QSlider *slider = it;
 
-			QString type, parameterName;
+			QString type;
+			QString parameterName;
 			GetNameAndType(name, &parameterName, &type);
 
 			if (type == QString("slider"))
 			{
 				QApplication::connect(slider, SIGNAL(sliderMoved(int)), this, SLOT(slotSliderMoved(int)));
 
-				QString spinBoxName = QString("spinbox_") + parameterName;
+				const QString spinBoxName = QString("spinbox_") + parameterName;
 				QDoubleSpinBox *spinBox = slider->parent()->findChild<QDoubleSpinBox *>(spinBoxName);
 				if (spinBox)
 				{
@@ -469,7 +479,7 @@ void cAutomatedWidgets::ConnectSignalsForSlidersInWindow(QWidget *window) const
 				else
 				{
 					qWarning() << "ConnectSignalsForSlidersInWindow() error: spinbox " << spinBoxName
-										 << " doesn't exists" << endl;
+										 << " doesn't exists";
 				}
 			}
 			if (type == QString("logslider"))
@@ -477,7 +487,7 @@ void cAutomatedWidgets::ConnectSignalsForSlidersInWindow(QWidget *window) const
 				QApplication::connect(
 					slider, SIGNAL(sliderMoved(int)), this, SLOT(slotLogSliderMoved(int)));
 
-				QString editFieldName = QString("logedit_") + parameterName;
+				const QString editFieldName = QString("logedit_") + parameterName;
 				QLineEdit *lineEdit = slider->parent()->findChild<QLineEdit *>(editFieldName);
 				if (lineEdit)
 				{
@@ -487,7 +497,7 @@ void cAutomatedWidgets::ConnectSignalsForSlidersInWindow(QWidget *window) const
 				else
 				{
 					qWarning() << "ConnectSignalsForSlidersInWindow() error: lineEdit " << editFieldName
-										 << " doesn't exists" << endl;
+										 << " doesn't exists";
 				}
 			}
 			if (type == QString("logslidervect3"))
@@ -495,7 +505,7 @@ void cAutomatedWidgets::ConnectSignalsForSlidersInWindow(QWidget *window) const
 				QApplication::connect(
 					slider, SIGNAL(sliderMoved(int)), this, SLOT(slotLogSliderVect3Moved(int)));
 
-				QString editFieldName = QString("logvect3_") + parameterName;
+				const QString editFieldName = QString("logvect3_") + parameterName;
 				QLineEdit *lineEdit = slider->parent()->findChild<QLineEdit *>(editFieldName);
 				if (lineEdit)
 				{
@@ -505,7 +515,7 @@ void cAutomatedWidgets::ConnectSignalsForSlidersInWindow(QWidget *window) const
 				else
 				{
 					qWarning() << "ConnectSignalsForSlidersInWindow() error: lineEdit " << editFieldName
-										 << " doesn't exists" << endl;
+										 << " doesn't exists";
 				}
 			}
 			if (type == QString("sliderInt"))
@@ -513,7 +523,7 @@ void cAutomatedWidgets::ConnectSignalsForSlidersInWindow(QWidget *window) const
 				QApplication::connect(
 					slider, SIGNAL(sliderMoved(int)), this, SLOT(slotIntSliderMoved(int)));
 
-				QString spinboxName = QString("spinboxInt_") + parameterName;
+				const QString spinboxName = QString("spinboxInt_") + parameterName;
 				QSpinBox *spinbox = slider->parent()->findChild<QSpinBox *>(spinboxName);
 				if (spinbox)
 				{
@@ -523,13 +533,13 @@ void cAutomatedWidgets::ConnectSignalsForSlidersInWindow(QWidget *window) const
 				else
 				{
 					qWarning() << "ConnectSignalsForSlidersInWindow() error: spinboxInt " << spinboxName
-										 << " doesn't exists" << endl;
+										 << " doesn't exists";
 				}
 			}
 			if (type == QString("slider3"))
 			{
 				QApplication::connect(slider, SIGNAL(sliderMoved(int)), this, SLOT(slotSlider3Moved(int)));
-				QString spinboxName = QString("spinbox3_") + parameterName;
+				const QString spinboxName = QString("spinbox3_") + parameterName;
 				QDoubleSpinBox *spinbox = slider->parent()->findChild<QDoubleSpinBox *>(spinboxName);
 				if (spinbox)
 				{
@@ -539,13 +549,13 @@ void cAutomatedWidgets::ConnectSignalsForSlidersInWindow(QWidget *window) const
 				else
 				{
 					qWarning() << "ConnectSignalsForSlidersInWindow() error: spinbox3 " << spinboxName
-										 << " doesn't exists" << endl;
+										 << " doesn't exists";
 				}
 			}
 			if (type == QString("slider4"))
 			{
 				QApplication::connect(slider, SIGNAL(valueChanged(int)), this, SLOT(slotSlider4Moved(int)));
-				QString spinboxName = QString("spinbox4_") + parameterName;
+				const QString spinboxName = QString("spinbox4_") + parameterName;
 				QDoubleSpinBox *spinbox = slider->parent()->findChild<QDoubleSpinBox *>(spinboxName);
 				if (spinbox)
 				{
@@ -555,28 +565,28 @@ void cAutomatedWidgets::ConnectSignalsForSlidersInWindow(QWidget *window) const
 				else
 				{
 					qWarning() << "ConnectSignalsForSlidersInWindow() error: spinbox4 " << spinboxName
-										 << " doesn't exists" << endl;
+										 << " doesn't exists";
 				}
 			}
 		}
 	}
 
 	QList<QDial *> widgetList2 = window->findChildren<QDial *>(QString());
-	QList<QDial *>::iterator it2;
-	for (it2 = widgetList2.begin(); it2 != widgetList2.end(); ++it2)
+	for (auto &it2 : widgetList2)
 	{
-		QString name = (*it2)->objectName();
-		if (name.length() > 1 && (*it2)->metaObject()->className() == QString("QDial"))
+		QString name = it2->objectName();
+		if (name.length() > 1 && it2->metaObject()->className() == QString("QDial"))
 		{
-			const QDial *dial = *it2;
-			QString type, parameterName;
+			const QDial *dial = it2;
+			QString type;
+			QString parameterName;
 			GetNameAndType(name, &parameterName, &type);
 
 			if (type == QString("dial3"))
 			{
-				QApplication::connect(dial, SIGNAL(sliderMoved(int)), this, SLOT(slotDial3Moved(int)));
+				QApplication::connect(dial, SIGNAL(valueChanged(int)), this, SLOT(slotDial3Moved(int)));
 
-				QString spinBoxName = QString("spinboxd3_") + parameterName;
+				const QString spinBoxName = QString("spinboxd3_") + parameterName;
 				QDoubleSpinBox *spinBox = dial->parent()->findChild<QDoubleSpinBox *>(spinBoxName);
 				if (spinBox)
 				{
@@ -586,14 +596,14 @@ void cAutomatedWidgets::ConnectSignalsForSlidersInWindow(QWidget *window) const
 				else
 				{
 					qWarning() << "ConnectSignalsForSlidersInWindow() error: spinboxd3 " << spinBoxName
-										 << " doesn't exists" << endl;
+										 << " doesn't exists";
 				}
 			}
 			if (type == QString("dial4"))
 			{
 				QApplication::connect(dial, SIGNAL(sliderMoved(int)), this, SLOT(slotDial4Moved(int)));
 
-				QString spinBoxName = QString("spinboxd4_") + parameterName;
+				const QString spinBoxName = QString("spinboxd4_") + parameterName;
 				QDoubleSpinBox *spinBox = dial->parent()->findChild<QDoubleSpinBox *>(spinBoxName);
 				if (spinBox)
 				{
@@ -603,14 +613,14 @@ void cAutomatedWidgets::ConnectSignalsForSlidersInWindow(QWidget *window) const
 				else
 				{
 					qWarning() << "ConnectSignalsForSlidersInWindow() error: spinboxd4 " << spinBoxName
-										 << " doesn't exists" << endl;
+										 << " doesn't exists";
 				}
 			}
 			if (type == QString("dial"))
 			{
 				QApplication::connect(dial, SIGNAL(sliderMoved(int)), this, SLOT(slotDialMoved(int)));
 
-				QString spinBoxName = QString("spinboxd_") + parameterName;
+				const QString spinBoxName = QString("spinboxd_") + parameterName;
 				QDoubleSpinBox *spinBox = dial->parent()->findChild<QDoubleSpinBox *>(spinBoxName);
 				if (spinBox)
 				{
@@ -620,7 +630,7 @@ void cAutomatedWidgets::ConnectSignalsForSlidersInWindow(QWidget *window) const
 				else
 				{
 					qWarning() << "ConnectSignalsForSlidersInWindow() error: spinboxd " << spinBoxName
-										 << " doesn't exists" << endl;
+										 << " doesn't exists";
 				}
 			}
 		}
