@@ -1,7 +1,7 @@
 /**
  * Mandelbulber v2, a 3D fractal generator       ,=#MKNmMMKmmßMNWy,
  *                                             ,B" ]L,,p%%%,,,§;, "K
- * Copyright (C) 2015-17 Mandelbulber Team     §R-==%w["'~5]m%=L.=~5N
+ * Copyright (C) 2015-20 Mandelbulber Team     §R-==%w["'~5]m%=L.=~5N
  *                                        ,=mm=§M ]=4 yJKA"/-Nsaj  "Bw,==,,
  * This file is part of Mandelbulber.    §R.r= jw",M  Km .mM  FW ",§=ß., ,TN
  *                                     ,4R =%["w[N=7]J '"5=],""]]M,w,-; T=]M
@@ -34,24 +34,31 @@
 
 #include "queue.hpp"
 
-#include "../qt/thumbnail_widget.h"
-#include "../src/interface.hpp"
-#include "../src/rendered_image_widget.hpp"
+#include <QApplication>
+
+#include "ui_dock_queue.h"
+
 #include "animation_frames.hpp"
 #include "cimage.hpp"
-#include "render_window.hpp"
 #include "error_message.hpp"
 #include "fractal_container.hpp"
+#include "global_data.hpp"
 #include "headless.h"
 #include "initparameters.hpp"
+#include "interface.hpp"
 #include "keyframes.hpp"
 #include "parameters.hpp"
-#include "preview_file_dialog.h"
 #include "render_queue.hpp"
+#include "render_window.hpp"
+#include "rendered_image_widget.hpp"
 #include "settings.hpp"
 #include "system.hpp"
-#include "ui_dock_queue.h"
-#include "dock_queue.h"
+#include "system_data.hpp"
+#include "system_directories.hpp"
+
+#include "qt/dock_queue.h"
+#include "qt/preview_file_dialog.h"
+#include "qt/thumbnail_widget.h"
 
 cQueue *gQueue = nullptr;
 
@@ -59,6 +66,9 @@ cQueue::cQueue(cInterface *_interface, const QString &_queueListFileName,
 	const QString &_queueFolder, QObject *parent)
 		: QObject(parent), mainInterface(_interface)
 {
+	queueFileWatcher = new QFileSystemWatcher(this);
+	queueFolderWatcher = new QFileSystemWatcher(this);
+
 	// initializes queue and create necessary files and folders
 	queueListFileName = _queueListFileName;
 	queueFolder = _queueFolder;
@@ -82,52 +92,47 @@ cQueue::cQueue(cInterface *_interface, const QString &_queueListFileName,
 	}
 
 	// watch queue folder and the queue file in the filesystem
-	queueFileWatcher.addPath(queueListFileName);
-	queueFolderWatcher.addPath(queueFolder);
-	QApplication::connect(&queueFileWatcher, SIGNAL(fileChanged(const QString &)), this,
-		SLOT(queueFileChanged(const QString &)));
-	QApplication::connect(&queueFolderWatcher, SIGNAL(directoryChanged(const QString &)), this,
-		SLOT(queueFolderChanged(const QString &)));
+	queueFileWatcher->addPath(queueListFileName);
+	queueFolderWatcher->addPath(queueFolder);
+
+	connect(queueFileWatcher, &QFileSystemWatcher::fileChanged, this, &cQueue::queueFileChanged);
+	connect(
+		queueFolderWatcher, &QFileSystemWatcher::directoryChanged, this, &cQueue::queueFolderChanged);
 
 	UpdateListFromQueueFile();
 	UpdateListFromFileSystem();
 
-	image = new cImage(200, 200);
-
 	if (mainInterface->mainWindow)
 	{
 		ui = mainInterface->mainWindow->GetWidgetDockQueue()->GetUi();
 		// Queue
-		QApplication::connect(ui->pushButton_queue_add_current_settings, SIGNAL(clicked()), this,
-			SLOT(slotQueueAddCurrentSettings()));
+		connect(ui->pushButton_queue_add_current_settings, &QPushButton::clicked, this,
+			&cQueue::slotQueueAddCurrentSettings);
+		QApplication::connect(ui->pushButton_queue_add_from_file, &QPushButton::clicked, this,
+			&cQueue::slotQueueAddFromFile);
+		QApplication::connect(ui->pushButton_queue_add_orphaned, &QPushButton::clicked, this,
+			&cQueue::slotQueueAddOrphaned);
+		QApplication::connect(ui->pushButton_queue_remove_orphaned, &QPushButton::clicked, this,
+			&cQueue::slotQueueRemoveOrphaned);
 		QApplication::connect(
-			ui->pushButton_queue_add_from_file, SIGNAL(clicked()), this, SLOT(slotQueueAddFromFile()));
+			ui->pushButton_queue_render_queue, &QPushButton::clicked, this, &cQueue::slotQueueRender);
 		QApplication::connect(
-			ui->pushButton_queue_add_orphaned, SIGNAL(clicked()), this, SLOT(slotQueueAddOrphaned()));
-		QApplication::connect(ui->pushButton_queue_remove_orphaned, SIGNAL(clicked()), this,
-			SLOT(slotQueueRemoveOrphaned()));
-		QApplication::connect(
-			ui->pushButton_queue_render_queue, SIGNAL(clicked()), this, SLOT(slotQueueRender()));
-		QApplication::connect(
-			ui->pushButton_queue_stop_rendering, SIGNAL(clicked()), this, SLOT(slotStopRequest()));
-		QApplication::connect(ui->checkBox_show_queue_thumbnails, SIGNAL(stateChanged(int)), this,
-			SLOT(slotShowQueueThumbsChanges(int)));
+			ui->pushButton_queue_stop_rendering, &QPushButton::clicked, this, &cQueue::slotStopRequest);
+		QApplication::connect(ui->checkBox_show_queue_thumbnails, &QCheckBox::stateChanged, this,
+			&cQueue::slotShowQueueThumbsChanges);
 
-		QApplication::connect(this, SIGNAL(queueChanged()), this, SLOT(slotQueueListUpdate()));
-		QApplication::connect(this, SIGNAL(queueChanged(int)), this, SLOT(slotQueueListUpdate(int)));
-		QApplication::connect(
-			this, SIGNAL(queueChanged(int, int)), this, SLOT(slotQueueListUpdate(int, int)));
+		QApplication::connect(this, &cQueue::queueChanged, this, &cQueue::slotQueueListUpdate);
+		QApplication::connect(this, &cQueue::queueChangedRow, this, &cQueue::slotQueueListUpdateRow);
+		QApplication::connect(this, &cQueue::queueChangedCell, this, &cQueue::slotQueueListUpdateCell);
 
-		renderedImageWidget = mainInterface->mainWindow->GetWidgetDockQueue()->GetRenderedImageWidget();
-		image->CreatePreview(1.0, 400, 300, renderedImageWidget);
-		renderedImageWidget->setMinimumSize(image->GetPreviewWidth(), image->GetPreviewHeight());
-		renderedImageWidget->AssignImage(image);
+		//		connect(ui->tableWidget_queue_list, SIGNAL(cellChanged(int, int)), this,
+		//			SLOT(slotQueueListUpdate(int, int)));
 
 		emit queueChanged();
 	}
 	else
 	{
-		renderedImageWidget = nullptr;
+		// noting
 	}
 
 	stopRequest = false;
@@ -135,7 +140,7 @@ cQueue::cQueue(cInterface *_interface, const QString &_queueListFileName,
 
 cQueue::~cQueue()
 {
-	delete image;
+	// nothing to delete
 }
 
 void cQueue::Append(const QString &filename, enumRenderType renderType)
@@ -162,8 +167,9 @@ void cQueue::Append(enumRenderType renderType)
 	Append(gPar, gParFractal, gAnimFrames, gKeyframes, renderType);
 }
 
-void cQueue::Append(cParameterContainer *par, cFractalContainer *fractPar, cAnimationFrames *frames,
-	cKeyframes *keyframes, enumRenderType renderType)
+void cQueue::Append(std::shared_ptr<cParameterContainer> par,
+	std::shared_ptr<cFractalContainer> fractPar, std::shared_ptr<cAnimationFrames> frames,
+	std::shared_ptr<cKeyframes> keyframes, enumRenderType renderType)
 {
 	// add settings to queue
 	cSettings parSettings(cSettings::formatCondensedText);
@@ -178,7 +184,6 @@ void cQueue::Append(cParameterContainer *par, cFractalContainer *fractPar, cAnim
 void cQueue::AppendList(const QString &filename)
 {
 	// add all entries from list given with filename to current list
-	// qDebug() << "AppendList: " << filename;
 	if (QFileInfo(filename).suffix() == QString("fractlist"))
 	{
 		mutex.lock();
@@ -236,8 +241,9 @@ bool cQueue::Get()
 	return Get(gPar, gParFractal, gAnimFrames, gKeyframes);
 }
 
-bool cQueue::Get(cParameterContainer *par, cFractalContainer *fractPar, cAnimationFrames *frames,
-	cKeyframes *keyframes)
+bool cQueue::Get(std::shared_ptr<cParameterContainer> par,
+	std::shared_ptr<cFractalContainer> fractPar, std::shared_ptr<cAnimationFrames> frames,
+	std::shared_ptr<cKeyframes> keyframes)
 {
 	// get next fractal from queue
 	structQueueItem queueItem = GetNextFromList();
@@ -328,7 +334,13 @@ void cQueue::SwapQueueItem(int i, int j)
 {
 	mutex.lock();
 	if (i >= 0 && j >= 0 && i < queueListFileName.size() - 1 && j < queueListFileName.size() - 1)
+	{
+#if QT_VERSION < QT_VERSION_CHECK(5, 13, 0)
 		queueListFromFile.swap(i, j);
+#else
+		queueListFromFile.swapItemsAt(i, j);
+#endif
+	}
 	mutex.unlock();
 	StoreList();
 }
@@ -347,9 +359,9 @@ void cQueue::RemoveQueueItem(const structQueueItem queueItem)
 	RemoveFromList(queueItem);
 	// check if fract file is still on the list (one file with different renderTypes)
 	mutex.lock();
-	for (int i = 0; i < queueListFromFile.size(); i++)
+	for (const auto &i : queueListFromFile)
 	{
-		if (queueListFromFile.at(i).filename == queueItem.filename)
+		if (i.filename == queueItem.filename)
 		{
 			mutex.unlock();
 			return;
@@ -396,10 +408,9 @@ void cQueue::StoreList()
 	{
 		QTextStream stream(&file);
 		stream << "#\n# Mandelbulber queue file\n#\n";
-		for (int i = 0; i < queueListFromFile.size(); i++)
+		for (const auto &i : queueListFromFile)
 		{
-			stream << queueListFromFile.at(i).filename << " "
-						 << GetTypeText(queueListFromFile.at(i).renderType) << endl;
+			stream << i.filename << " " << GetTypeText(i.renderType) << "\n";
 		}
 	}
 	file.close();
@@ -464,7 +475,6 @@ QString cQueue::GetTypeColor(enumRenderType queueType)
 
 void cQueue::queueFileChanged(const QString &path)
 {
-	// qDebug() << "queueFileChanged";
 	if (path == queueListFileName)
 	{
 		UpdateListFromQueueFile();
@@ -473,7 +483,6 @@ void cQueue::queueFileChanged(const QString &path)
 
 void cQueue::queueFolderChanged(const QString &path)
 {
-	// qDebug() << "queueFolderChanged";
 	if (path == queueFolder)
 	{
 		UpdateListFromFileSystem();
@@ -524,33 +533,40 @@ void cQueue::UpdateListFromFileSystem()
 void cQueue::RenderQueue() const
 {
 	QThread *thread = new QThread; // deleted by deleteLater()
-	cRenderQueue *renderQueue = new cRenderQueue(image, renderedImageWidget);
+	cRenderQueue *renderQueue =
+		new cRenderQueue(mainInterface->mainImage, mainInterface->renderedImage);
 	renderQueue->moveToThread(thread);
 	renderQueue->setObjectName("Queue");
-	QObject::connect(thread, SIGNAL(started()), renderQueue, SLOT(slotRenderQueue()));
-	QObject::connect(renderQueue, SIGNAL(finished()), renderQueue, SLOT(deleteLater()));
+	connect(thread, SIGNAL(started()), renderQueue, SLOT(slotRenderQueue()));
+	connect(renderQueue, SIGNAL(finished()), renderQueue, SLOT(deleteLater()));
 	if (gMainInterface->mainWindow)
 	{
-		QObject::connect(renderQueue,
+		connect(renderQueue,
 			SIGNAL(updateProgressAndStatus(QString, QString, double, cProgressText::enumProgressType)),
 			gMainInterface->mainWindow,
 			SLOT(slotUpdateProgressAndStatus(QString, QString, double, cProgressText::enumProgressType)));
-		QObject::connect(renderQueue, SIGNAL(updateProgressHide(cProgressText::enumProgressType)),
+		connect(renderQueue, SIGNAL(updateProgressHide(cProgressText::enumProgressType)),
 			gMainInterface->mainWindow, SLOT(slotUpdateProgressHide(cProgressText::enumProgressType)));
+		connect(renderQueue, SIGNAL(updateImage()), mainInterface->renderedImage, SLOT(update()));
 	}
-	if (gMainInterface->headless)
+	if (mainInterface->headless)
 	{
-		QObject::connect(renderQueue,
+		connect(renderQueue,
 			SIGNAL(updateProgressAndStatus(QString, QString, double, cProgressText::enumProgressType)),
 			gMainInterface->headless,
 			SLOT(slotUpdateProgressAndStatus(QString, QString, double, cProgressText::enumProgressType)));
-		QObject::connect(renderQueue, SIGNAL(updateStatistics(cStatistics)), gMainInterface->headless,
+		connect(renderQueue, SIGNAL(updateStatistics(cStatistics)), mainInterface->headless,
 			SLOT(slotUpdateStatistics(cStatistics)));
 	}
 	QObject::connect(renderQueue, SIGNAL(finished()), thread, SLOT(quit()));
 	QObject::connect(thread, SIGNAL(finished()), thread, SLOT(deleteLater()));
 	thread->setObjectName("RenderQueue");
 
+	// this is needed to get settings from main parameter container.
+	// Further will be used only local container
+	systemData.numberOfThreads = gPar->Get<int>("limit_CPU_cores");
+	systemData.threadsPriority = enumRenderingThreadPriority(gPar->Get<int>("threads_priority"));
+
 	thread->start();
 }
 
@@ -570,7 +586,7 @@ void cQueue::slotQueueRender() const
 	else
 	{
 		cErrorMessage::showMessage(
-			QObject::tr("No queue items to render"), cErrorMessage::errorMessage, nullptr);
+			QObject::tr("No queue items to render"), cErrorMessage::warningMessage, nullptr);
 	}
 }
 
@@ -587,7 +603,7 @@ void cQueue::slotQueueAddFromFile()
 	dialog.setOption(QFileDialog::DontUseNativeDialog);
 	dialog.setFileMode(QFileDialog::ExistingFile);
 	dialog.setNameFilter(tr("Fractals (*.txt *.fract)"));
-	dialog.setDirectory(systemData.GetSettingsFolder() + QDir::separator());
+	dialog.setDirectory(systemDirectories.GetSettingsFolder() + QDir::separator());
 	dialog.selectFile(systemData.lastSettingsFile);
 	dialog.setAcceptMode(QFileDialog::AcceptOpen);
 	dialog.setWindowTitle(tr("Add file to queue..."));
@@ -612,25 +628,25 @@ void cQueue::slotQueueRemoveOrphaned()
 
 void cQueue::slotQueueRemoveItem()
 {
-	QString buttonName = this->sender()->objectName();
+	QString buttonName = sender()->objectName();
 	RemoveQueueItem(buttonName.toInt());
 }
 
 void cQueue::slotQueueMoveItemUp()
 {
-	QString buttonName = this->sender()->objectName();
+	QString buttonName = sender()->objectName();
 	SwapQueueItem(buttonName.toInt(), buttonName.toInt() - 1);
 }
 
 void cQueue::slotQueueMoveItemDown()
 {
-	QString buttonName = this->sender()->objectName();
+	QString buttonName = sender()->objectName();
 	SwapQueueItem(buttonName.toInt(), buttonName.toInt() + 1);
 }
 
 void cQueue::slotQueueTypeChanged(int index)
 {
-	QString buttonName = this->sender()->objectName();
+	QString buttonName = sender()->objectName();
 	UpdateQueueItemType(buttonName.toInt(), cQueue::enumRenderType(index));
 }
 
@@ -663,22 +679,23 @@ void cQueue::slotQueueListUpdate()
 	// update table
 	for (int i = 0; i < table->rowCount(); i++)
 	{
-		slotQueueListUpdate(i);
+		emit queueChangedRow(i);
+		// gApplication->processEvents();
 	}
 }
 
-void cQueue::slotQueueListUpdate(int i)
+void cQueue::slotQueueListUpdateRow(int i)
 {
 	// update row i
 	QTableWidget *table = ui->tableWidget_queue_list;
 	table->setRowHeight(i, 70);
 	for (int j = 0; j < table->columnCount(); j++)
 	{
-		slotQueueListUpdate(i, j);
+		emit queueChangedCell(i, j);
 	}
 }
 
-void cQueue::slotQueueListUpdate(int i, int j)
+void cQueue::slotQueueListUpdateCell(int i, int j)
 {
 	// update element in row i, column j
 	QTableWidget *table = ui->tableWidget_queue_list;
@@ -690,84 +707,94 @@ void cQueue::slotQueueListUpdate(int i, int j)
 		table->setItem(i, j, cell);
 	}
 	QList<cQueue::structQueueItem> queueList = GetListFromQueueFile();
-	switch (j)
+
+	if (i < queueList.size())
 	{
-		case 0: cell->setText(queueList.at(i).filename); break;
-		case 1:
+		switch (j)
 		{
-			if (ui->checkBox_show_queue_thumbnails->isChecked())
+			case 0: cell->setText(queueList.at(i).filename); break;
+			case 1:
 			{
-				cParameterContainer tempPar = *gPar;
-				cFractalContainer tempFract = *gParFractal;
-				cSettings parSettings(cSettings::formatFullText);
-				parSettings.BeQuiet(true);
-				if (parSettings.LoadFromFile(queueList.at(i).filename)
-						&& parSettings.Decode(&tempPar, &tempFract))
+				if (ui->checkBox_show_queue_thumbnails->isChecked())
 				{
-					cThumbnailWidget *thumbWidget = static_cast<cThumbnailWidget *>(table->cellWidget(i, j));
-					if (!thumbWidget)
+					std::shared_ptr<cParameterContainer> tempPar(new cParameterContainer());
+					std::shared_ptr<cFractalContainer> tempFract(new cFractalContainer());
+					InitParams(tempPar);
+					for (int f = 0; f < NUMBER_OF_FRACTALS; f++)
+						InitFractalParams(tempFract->at(f));
+					InitMaterialParams(1, tempPar);
+
+					cSettings parSettings(cSettings::formatFullText);
+					parSettings.BeQuiet(true);
+					if (parSettings.LoadFromFile(queueList.at(i).filename)
+							&& parSettings.Decode(tempPar, tempFract))
 					{
-						thumbWidget = new cThumbnailWidget(100, 70, 1, table);
-						thumbWidget->UseOneCPUCore(true);
-						thumbWidget->AssignParameters(tempPar, tempFract);
-						table->setCellWidget(i, j, thumbWidget);
-					}
-					else
-					{
-						thumbWidget->AssignParameters(tempPar, tempFract);
-						thumbWidget->update();
+						cThumbnailWidget *thumbWidget =
+							static_cast<cThumbnailWidget *>(table->cellWidget(i, j));
+						if (!thumbWidget)
+						{
+							thumbWidget = new cThumbnailWidget(100, 70, 1, table);
+							thumbWidget->UseOneCPUCore(true);
+							thumbWidget->AssignParameters(tempPar, tempFract);
+							table->setCellWidget(i, j, thumbWidget);
+						}
+						else
+						{
+							thumbWidget->AssignParameters(tempPar, tempFract);
+							thumbWidget->update();
+						}
 					}
 				}
+				break;
 			}
-			break;
+			case 2:
+			{
+				QComboBox *typeComboBox = new QComboBox;
+				typeComboBox->addItem(cQueue::GetTypeName(cQueue::queue_STILL));
+				typeComboBox->addItem(cQueue::GetTypeName(cQueue::queue_FLIGHT));
+				typeComboBox->addItem(cQueue::GetTypeName(cQueue::queue_KEYFRAME));
+				typeComboBox->setCurrentIndex(queueList.at(i).renderType);
+
+				typeComboBox->setObjectName(QString::number(i));
+				QObject::connect(
+					typeComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(slotQueueTypeChanged(int)));
+				table->setCellWidget(i, j, typeComboBox);
+				// cell->setTextColor(color);
+				break;
+			}
+			case 3:
+			{
+				QFrame *frame = new QFrame;
+				QGridLayout *gridLayout = new QGridLayout;
+				QToolButton *actionDelete = new QToolButton;
+				QToolButton *actionMoveUp = new QToolButton;
+				QToolButton *actionMoveDown = new QToolButton;
+
+				actionDelete->setIcon(actionDelete->style()->standardIcon(QStyle::SP_TrashIcon));
+				actionMoveUp->setIcon(actionMoveUp->style()->standardIcon(QStyle::SP_ArrowUp));
+				actionMoveDown->setIcon(actionMoveDown->style()->standardIcon(QStyle::SP_ArrowDown));
+
+				actionDelete->setObjectName(QString::number(i));
+				actionMoveUp->setObjectName(QString::number(i));
+				actionMoveDown->setObjectName(QString::number(i));
+
+				QObject::connect(actionDelete, SIGNAL(clicked()), this, SLOT(slotQueueRemoveItem()));
+				QObject::connect(actionMoveUp, SIGNAL(clicked()), this, SLOT(slotQueueMoveItemUp()));
+				QObject::connect(actionMoveDown, SIGNAL(clicked()), this, SLOT(slotQueueMoveItemDown()));
+
+				gridLayout->addWidget(actionMoveUp, 0, 0);
+				gridLayout->addWidget(actionMoveDown, 1, 0);
+				gridLayout->addWidget(actionDelete, 0, 1);
+				gridLayout->setSpacing(0);
+
+				if (i == 0) actionMoveUp->setEnabled(false);
+				if (i == queueList.size() - 1) actionMoveDown->setEnabled(false);
+				frame->setLayout(gridLayout);
+				table->setCellWidget(i, j, frame);
+				break;
+			}
+			default: break;
 		}
-		case 2:
-		{
-			QComboBox *typeComboBox = new QComboBox;
-			typeComboBox->addItem(cQueue::GetTypeName(cQueue::queue_STILL));
-			typeComboBox->addItem(cQueue::GetTypeName(cQueue::queue_FLIGHT));
-			typeComboBox->addItem(cQueue::GetTypeName(cQueue::queue_KEYFRAME));
-			typeComboBox->setCurrentIndex(queueList.at(i).renderType);
-
-			typeComboBox->setObjectName(QString::number(i));
-			QObject::connect(
-				typeComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(slotQueueTypeChanged(int)));
-			table->setCellWidget(i, j, typeComboBox);
-			// cell->setTextColor(color);
-			break;
-		}
-		case 3:
-		{
-			QFrame *frame = new QFrame;
-			QGridLayout *gridlayout = new QGridLayout;
-			QToolButton *actionDelete = new QToolButton;
-			QToolButton *actionMoveUp = new QToolButton;
-			QToolButton *actionMoveDown = new QToolButton;
-
-			actionDelete->setIcon(actionDelete->style()->standardIcon(QStyle::SP_TrashIcon));
-			actionMoveUp->setIcon(actionMoveUp->style()->standardIcon(QStyle::SP_ArrowUp));
-			actionMoveDown->setIcon(actionMoveDown->style()->standardIcon(QStyle::SP_ArrowDown));
-
-			actionDelete->setObjectName(QString::number(i));
-			actionMoveUp->setObjectName(QString::number(i));
-			actionMoveDown->setObjectName(QString::number(i));
-
-			QObject::connect(actionDelete, SIGNAL(clicked()), this, SLOT(slotQueueRemoveItem()));
-			QObject::connect(actionMoveUp, SIGNAL(clicked()), this, SLOT(slotQueueMoveItemUp()));
-			QObject::connect(actionMoveDown, SIGNAL(clicked()), this, SLOT(slotQueueMoveItemDown()));
-
-			gridlayout->addWidget(actionMoveUp, 0, 0);
-			gridlayout->addWidget(actionMoveDown, 1, 0);
-			gridlayout->addWidget(actionDelete, 0, 1);
-			gridlayout->setSpacing(0);
-
-			if (i == 0) actionMoveUp->setEnabled(false);
-			if (i == queueList.size() - 1) actionMoveDown->setEnabled(false);
-			frame->setLayout(gridlayout);
-			table->setCellWidget(i, j, frame);
-			break;
-		}
-		default: break;
 	}
 	table->blockSignals(false);
 }
