@@ -1,7 +1,7 @@
 /**
  * Mandelbulber v2, a 3D fractal generator       ,=#MKNmMMKmmßMNWy,
  *                                             ,B" ]L,,p%%%,,,§;, "K
- * Copyright (C) 2016-17 Mandelbulber Team     §R-==%w["'~5]m%=L.=~5N
+ * Copyright (C) 2016-24 Mandelbulber Team     §R-==%w["'~5]m%=L.=~5N
  *                                        ,=mm=§M ]=4 yJKA"/-Nsaj  "Bw,==,,
  * This file is part of Mandelbulber.    §R.r= jw",M  Km .mM  FW ",§=ß., ,TN
  *                                     ,4R =%["w[N=7]J '"5=],""]]M,w,-; T=]M
@@ -31,30 +31,37 @@
  *
  * RenderWindow class - main program window
  *
- * This file contains implementation of the miscellanious slots in RenderWindow class.
- * See also header render_window.hpp and whole implementation of class
+ * This file contains implementation of the miscellaneous slots in RenderWindow
+ * class. See also header render_window.hpp and whole implementation of class
  * spread over render_window_*.cpp
  */
 
-#include "../qt/material_editor.h"
-#include "render_window.hpp"
-#include "../qt/preferences_dialog.h"
-#include "../qt/thumbnail_widget.h"
-#include "../qt/voxel_export_dialog.h"
-#include "../qt/mesh_export_dialog.h"
-#include "../src/material_item_model.h"
+#include "ui_render_window.h"
+
+#include "animation_flight.hpp"
 #include "animation_frames.hpp"
+#include "animation_keyframes.hpp"
 #include "cimage.hpp"
+#include "global_data.hpp"
 #include "image_scale.hpp"
 #include "initparameters.hpp"
 #include "interface.hpp"
 #include "keyframes.hpp"
+#include "manipulations.h"
+#include "material_item_model.h"
+#include "render_window.hpp"
 #include "settings.hpp"
-#include "system.hpp"
-#include "animation_flight.hpp"
-#include "animation_keyframes.hpp"
-#include "ui_render_window.h"
-#include "global_data.hpp"
+#include "shortcuts.h"
+#include "system_data.hpp"
+#include "system_directories.hpp"
+#include "write_log.hpp"
+
+#include "qt/detached_window.h"
+#include "qt/material_editor.h"
+#include "qt/mesh_export_dialog.h"
+#include "qt/preferences_dialog.h"
+#include "qt/thumbnail_widget.h"
+#include "qt/voxel_export_dialog.h"
 
 void RenderWindow::slotResizedScrolledAreaImage(int width, int height) const
 {
@@ -85,8 +92,11 @@ void RenderWindow::slotChangedComboImageScale(int index) const
 		gMainInterface->mainImage->CreatePreview(
 			scale, areaWidth, areaHeight, gMainInterface->renderedImage);
 		gMainInterface->mainImage->UpdatePreview();
+
+		double dpiScale = devicePixelRatioF();
 		gMainInterface->renderedImage->setMinimumSize(
-			gMainInterface->mainImage->GetPreviewWidth(), gMainInterface->mainImage->GetPreviewHeight());
+			gMainInterface->mainImage->GetPreviewWidth() / dpiScale,
+			gMainInterface->mainImage->GetPreviewHeight() / dpiScale);
 
 		gPar->Set("image_preview_scale", index);
 	}
@@ -116,8 +126,12 @@ void RenderWindow::slotMouseClickOnImage(int x, int y, Qt::MouseButton button) c
 		case RenderedImage::clickPlacePrimitive:
 		case RenderedImage::clickPlaceRandomLightCenter:
 		case RenderedImage::clickGetPoint:
+		case RenderedImage::clickWrapLimitsAroundObject:
 		{
-			gMainInterface->SetByMouse(CVector2<double>(x, y), button, mode);
+			if (gMainInterface->renderedImage->GetEnableClickModes())
+			{
+				manipulations->SetByMouse(CVector2<double>(x, y), button, mode);
+			}
 			break;
 		}
 		case RenderedImage::clickDoNothing:
@@ -127,11 +141,45 @@ void RenderWindow::slotMouseClickOnImage(int x, int y, Qt::MouseButton button) c
 	}
 }
 
+void RenderWindow::slotMouseDragStart(int x, int y, Qt::MouseButtons buttons)
+{
+	int index = ui->comboBox_mouse_click_function->currentIndex();
+	QList<QVariant> mode = ui->comboBox_mouse_click_function->itemData(index).toList();
+	RenderedImage::enumClickMode clickMode = RenderedImage::enumClickMode(mode.at(0).toInt());
+
+	switch (clickMode)
+	{
+		case RenderedImage::clickMoveCamera:
+		case RenderedImage::clickPlaceLight:
+		case RenderedImage::clickPlacePrimitive:
+		{
+			if (gMainInterface->renderedImage->GetEnableClickModes())
+			{
+				manipulations->MouseDragStart(CVector2<double>(x, y), buttons, mode);
+			}
+			break;
+		}
+		default:
+			// nothing
+			break;
+	}
+}
+
+void RenderWindow::slotMouseDragFinish()
+{
+	manipulations->MouseDragFinish();
+}
+
+void RenderWindow::slotMouseDragDelta(int dx, int dy)
+{
+	manipulations->MouseDragDelta(dx, dy);
+}
+
 void RenderWindow::slotChangedComboMouseClickFunction(int index) const
 {
 	if (index >= 0) // if list is empty, then index = -1
 	{
-		QComboBox *comboBox = static_cast<QComboBox *>(this->sender());
+		QComboBox *comboBox = static_cast<QComboBox *>(sender());
 		QList<QVariant> item = comboBox->itemData(index).toList();
 		gMainInterface->renderedImage->setClickMode(item);
 	}
@@ -139,70 +187,180 @@ void RenderWindow::slotChangedComboMouseClickFunction(int index) const
 
 void RenderWindow::slotKeyPressOnImage(QKeyEvent *event)
 {
-
-	int key = event->key();
-	Qt::KeyboardModifiers modifiers = event->modifiers();
-	if (modifiers & Qt::ShiftModifier)
-	{ // Shift pressed
-		switch (key)
-		{
-			case Qt::Key_Up: gMainInterface->RotateCamera("bu_rotate_up"); break;
-			case Qt::Key_Down: gMainInterface->RotateCamera("bu_rotate_down"); break;
-			case Qt::Key_Left: gMainInterface->RotateCamera("bu_rotate_left"); break;
-			case Qt::Key_Right: gMainInterface->RotateCamera("bu_rotate_right"); break;
-			default: break;
-		}
-	}
-	else if (modifiers & Qt::ControlModifier)
-	{ // Ctrl pressed
-		switch (key)
-		{
-			case Qt::Key_Up: gMainInterface->MoveCamera("bu_move_up"); break;
-			case Qt::Key_Down: gMainInterface->MoveCamera("bu_move_down"); break;
-			case Qt::Key_Left: gMainInterface->RotateCamera("bu_rotate_roll_left"); break;
-			case Qt::Key_Right: gMainInterface->RotateCamera("bu_rotate_roll_right"); break;
-			default: break;
-		}
-	}
-	else
-	{
-		// No keyboard modifiers
-		switch (key)
-		{
-			case Qt::Key_W: gMainInterface->MoveCamera("bu_move_up"); break;
-			case Qt::Key_S: gMainInterface->MoveCamera("bu_move_down"); break;
-			case Qt::Key_A: gMainInterface->MoveCamera("bu_move_left"); break;
-			case Qt::Key_D: gMainInterface->MoveCamera("bu_move_right"); break;
-			case Qt::Key_Up: gMainInterface->MoveCamera("bu_move_forward"); break;
-			case Qt::Key_Down: gMainInterface->MoveCamera("bu_move_backward"); break;
-			case Qt::Key_Left: gMainInterface->MoveCamera("bu_move_left"); break;
-			case Qt::Key_Right: gMainInterface->MoveCamera("bu_move_right"); break;
-			default: break;
-		}
-	}
+	currentKeyEvents.append(event->key());
+	lastKeyEventModifiers = event->modifiers();
+	slotKeyHandle();
+	buttonPressTimer->start();
 }
 
 void RenderWindow::slotKeyReleaseOnImage(QKeyEvent *event)
 {
-	(void)event;
+	currentKeyEvents.removeOne(event->key());
+
+	lastKeyEventModifiers = event->modifiers();
+	slotKeyHandle();
+	buttonPressTimer->stop();
 }
 
-void RenderWindow::slotMouseWheelRotatedOnImage(int delta) const
+void RenderWindow::slotButtonLongPress()
 {
-	int index = ui->comboBox_mouse_click_function->currentIndex();
-	QList<QVariant> mode = ui->comboBox_mouse_click_function->itemData(index).toList();
-	RenderedImage::enumClickMode clickMode = RenderedImage::enumClickMode(mode.at(0).toInt());
-	switch (clickMode)
+	slotKeyHandle();
+}
+
+void RenderWindow::slotKeyHandle()
+{
+	if (currentKeyEvents.size() == 0) return;
+
+	gMainInterface->SynchronizeInterface(gPar, gParFractal, qInterface::read);
+
+	bool render = false;
+
+	for (int i = 0; i < currentKeyEvents.size(); i++)
 	{
-		case RenderedImage::clickPlaceLight:
+		enumShortcuts shortcut =
+			cShortcuts::getShortcut(gPar.get(), currentKeyEvents.at(i), lastKeyEventModifiers);
+
+		switch (shortcut)
 		{
-			double deltaLog = exp(delta * 0.001);
-			double dist = ui->widgetEffects->GetAuxLightManualPlacementDistance();
-			dist *= deltaLog;
-			ui->widgetEffects->SetAuxLightManualPlacementDistance(dist);
-			break;
+			case enumShortcuts::moveForward:
+				manipulations->MoveCamera("bu_move_forward");
+				render = true;
+				break;
+			case enumShortcuts::moveBackward:
+				manipulations->MoveCamera("bu_move_backward");
+				render = true;
+				break;
+			case enumShortcuts::moveLeft:
+				manipulations->MoveCamera("bu_move_left");
+				render = true;
+				break;
+			case enumShortcuts::moveRight:
+				manipulations->MoveCamera("bu_move_right");
+				render = true;
+				break;
+			case enumShortcuts::moveUp:
+				manipulations->MoveCamera("bu_move_up");
+				render = true;
+				break;
+			case enumShortcuts::moveDown:
+				manipulations->MoveCamera("bu_move_down");
+				render = true;
+				break;
+			case enumShortcuts::rotateLeft:
+				manipulations->RotateCamera("bu_rotate_left");
+				render = true;
+				break;
+			case enumShortcuts::rotateRight:
+				manipulations->RotateCamera("bu_rotate_right");
+				render = true;
+				break;
+			case enumShortcuts::rotateUp:
+				manipulations->RotateCamera("bu_rotate_up");
+				render = true;
+				break;
+			case enumShortcuts::rotateDown:
+				manipulations->RotateCamera("bu_rotate_down");
+				render = true;
+				break;
+			case enumShortcuts::rollLeft:
+				manipulations->RotateCamera("bu_rotate_roll_left");
+				render = true;
+				break;
+			case enumShortcuts::rollRight:
+				manipulations->RotateCamera("bu_rotate_roll_right");
+				render = true;
+				break;
+			case enumShortcuts::addKeyframe:
+				currentKeyEvents.removeOne(currentKeyEvents.at(i)); // long press not allowed
+				gKeyframeAnimation->slotAddKeyframe();
+				break;
+			case enumShortcuts::modifyKeyframe:
+				currentKeyEvents.removeOne(currentKeyEvents.at(i)); // long press not allowed
+				gKeyframeAnimation->slotModifyKeyframe();
+				break;
+			case enumShortcuts::render:
+				currentKeyEvents.removeOne(currentKeyEvents.at(i)); // long press not allowed
+				gMainInterface->StartRender();
+				break;
+			case enumShortcuts::stop:
+				currentKeyEvents.removeOne(currentKeyEvents.at(i)); // long press not allowed
+				gMainInterface->StopRender();
+				break;
+
+			default: break;
+		}
+	}
+
+	gMainInterface->SynchronizeInterface(gPar, gParFractal, qInterface::write);
+
+	if (render)
+	{
+		gMainInterface->StartRender();
+	}
+}
+
+// global shortcuts
+void RenderWindow::keyPressEvent(QKeyEvent *event)
+{
+	int key = event->key();
+	Qt::KeyboardModifiers modifiers = event->modifiers();
+
+	if (modifiers & Qt::ShiftModifier)
+	{
+		switch (key)
+		{
+			case Qt::Key_Escape: gMainInterface->GlobalStopRequest(); break;
+			default: break;
+		}
+	}
+}
+
+void RenderWindow::slotMouseWheelRotatedWithKeyOnImage(
+	int x, int y, int delta, Qt::KeyboardModifiers keyModifiers) const
+{
+	if (gMainInterface->renderedImage->GetEnableClickModes())
+	{
+		int index = ui->comboBox_mouse_click_function->currentIndex();
+		QList<QVariant> mode = ui->comboBox_mouse_click_function->itemData(index).toList();
+		RenderedImage::enumClickMode clickMode = RenderedImage::enumClickMode(mode.at(0).toInt());
+		switch (clickMode)
+		{
+			case RenderedImage::clickPlaceLight:
+			{
+				if (keyModifiers & Qt::AltModifier)
+				{
+					double deltaLog = exp(delta * 0.001);
+					double dist = ui->widgetEffects->GetAuxLightManualPlacementDistance();
+					dist *= deltaLog;
+					ui->widgetEffects->slotSetAuxLightManualPlacementDistance(dist);
+				}
+				else if (keyModifiers == Qt::NoModifier)
+				{
+					manipulations->MoveLightByWheel(delta);
+				}
+				break;
+			}
+			case RenderedImage::clickPlacePrimitive:
+			{
+				if (keyModifiers == Qt::NoModifier)
+				{
+					manipulations->MovePrimitiveByWheel(delta);
+				}
+				break;
+			}
+			case RenderedImage::clickMoveCamera:
+			{
+				Qt::MouseButton button = (delta > 0) ? Qt::LeftButton : Qt::RightButton;
+				if (ui->toolButton_preciseRotation->isChecked())
+				{
+					delta /= 10.0;
+				}
+				mode.append(QVariant(delta));
+				manipulations->SetByMouse(CVector2<double>(x, y), button, mode);
+				break;
+			}
+			default: break;
 		}
-		default: break;
 	}
 }
 
@@ -215,7 +373,7 @@ void RenderWindow::slotChangedCheckBoxCursorVisibility(int state)
 void RenderWindow::slotPopulateToolbar(bool completeRefresh)
 {
 	WriteLog("cInterface::PopulateToolbar(QWidget *window, QToolBar *toolBar) started", 2);
-	QDir toolbarDir = QDir(systemData.GetToolbarFolder());
+	QDir toolbarDir = QDir(gPar->Get<QString>("toolbar_path"));
 	toolbarDir.setSorting(QDir::Time);
 	QStringList toolbarFiles = toolbarDir.entryList(QDir::NoDotAndDotDot | QDir::Files);
 	QSignalMapper *mapPresetsFromExamplesLoad = new QSignalMapper(this);
@@ -225,9 +383,17 @@ void RenderWindow::slotPopulateToolbar(bool completeRefresh)
 
 	QList<QAction *> actions = ui->toolBar->actions();
 	QStringList toolbarInActions;
-	for (int i = 0; i < actions.size(); i++)
+
+#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
+	QApplication::connect(
+		mapPresetsFromExamplesLoad, SIGNAL(mapped(QString)), this, SLOT(slotMenuLoadPreset(QString)));
+#else
+	QApplication::connect(mapPresetsFromExamplesLoad, SIGNAL(mappedString(QString)), this,
+		SLOT(slotMenuLoadPreset(QString)));
+#endif
+
+	for (auto action : actions)
 	{
-		QAction *action = actions.at(i);
 		if (action->objectName() == "actionAdd_Settings_to_Toolbar") continue;
 		if (!toolbarFiles.contains(action->objectName()) || completeRefresh)
 		{
@@ -242,12 +408,14 @@ void RenderWindow::slotPopulateToolbar(bool completeRefresh)
 
 	for (int i = 0; i < toolbarFiles.size(); i++)
 	{
+		if (systemData.globalStopRequest) break;
+
 		if (toolbarInActions.contains(toolbarFiles.at(i)))
 		{
 			// already present
 			continue;
 		}
-		QString filename = systemData.GetToolbarFolder() + QDir::separator() + toolbarFiles.at(i);
+		QString filename = gPar->Get<QString>("toolbar_path") + QDir::separator() + toolbarFiles.at(i);
 		cThumbnailWidget *thumbWidget = nullptr;
 
 		if (QFileInfo(filename).suffix() == QString("fract"))
@@ -258,11 +426,11 @@ void RenderWindow::slotPopulateToolbar(bool completeRefresh)
 
 			if (parSettings.LoadFromFile(filename))
 			{
-				cParameterContainer *par = new cParameterContainer;
-				cFractalContainer *parFractal = new cFractalContainer;
+				std::shared_ptr<cParameterContainer> par(new cParameterContainer);
+				std::shared_ptr<cFractalContainer> parFractal(new cFractalContainer);
 				InitParams(par);
 				for (int j = 0; j < NUMBER_OF_FRACTALS; j++)
-					InitFractalParams(&parFractal->at(j));
+					InitFractalParams(parFractal->at(j));
 
 				/****************** TEMPORARY CODE FOR MATERIALS *******************/
 
@@ -274,11 +442,16 @@ void RenderWindow::slotPopulateToolbar(bool completeRefresh)
 				{
 					thumbWidget = new cThumbnailWidget(
 						gPar->Get<int>("toolbar_icon_size"), gPar->Get<int>("toolbar_icon_size"), 2, this);
+					par->Set("image_width", gPar->Get<int>("toolbar_icon_size") * 2);
+					par->Set("image_height", gPar->Get<int>("toolbar_icon_size") * 2);
+
 					thumbWidget->UseOneCPUCore(true);
-					thumbWidget->AssignParameters(*par, *parFractal);
+					par->Set("opencl_mode", gPar->Get<int>("opencl_mode"));
+					par->Set("opencl_enabled", gPar->Get<bool>("opencl_enabled"));
+					if (!par->Get<bool>("DOF_monte_carlo")) par->Set("opencl_mode", 0);
+					if (!gPar->Get<bool>("thumbnails_with_opencl")) par->Set("opencl_enabled", false);
+					thumbWidget->AssignParameters(par, parFractal);
 				}
-				delete par;
-				delete parFractal;
 			}
 		}
 
@@ -310,13 +483,19 @@ void RenderWindow::slotPopulateToolbar(bool completeRefresh)
 		}
 		QApplication::processEvents();
 	}
-	QApplication::connect(
-		mapPresetsFromExamplesLoad, SIGNAL(mapped(QString)), this, SLOT(slotMenuLoadPreset(QString)));
 
+#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
 	QApplication::connect(mapPresetsFromExamplesRemove, SIGNAL(mapped(QString)), this,
 		SLOT(slotMenuRemovePreset(QString)));
+#else
+	QApplication::connect(mapPresetsFromExamplesRemove, SIGNAL(mappedString(QString)), this,
+		SLOT(slotMenuRemovePreset(QString)));
+#endif
 
-	WriteLog("cInterface::PopulateToolbar(QWidget *window, QToolBar *toolBar) finished", 2);
+	WriteLog(
+		"cInterface::PopulateToolbar(QWidget *window, QToolBar *toolBar) "
+		"finished",
+		2);
 }
 
 void RenderWindow::slotPresetAddToToolbar()
@@ -325,7 +504,7 @@ void RenderWindow::slotPresetAddToToolbar()
 	gMainInterface->SynchronizeInterface(gPar, gParFractal, qInterface::read);
 	parSettings.CreateText(gPar, gParFractal, gAnimFrames, gKeyframes);
 	QString filename =
-		systemData.GetToolbarFolder() + QDir::separator() + parSettings.GetHashCode() + ".fract";
+		systemDirectories.GetToolbarFolder() + QDir::separator() + parSettings.GetHashCode() + ".fract";
 	parSettings.SaveToFile(filename);
 	slotPopulateToolbar();
 }
@@ -333,13 +512,17 @@ void RenderWindow::slotPresetAddToToolbar()
 void RenderWindow::slotMenuLoadPreset(QString filename)
 {
 	cSettings parSettings(cSettings::formatFullText);
+	gMainInterface->DisablePeriodicRefresh();
 	parSettings.LoadFromFile(filename);
 	gInterfaceReadyForSynchronization = false;
 	parSettings.Decode(gPar, gParFractal, gAnimFrames, gKeyframes);
-	gMainInterface->RebuildPrimitives(gPar);
 	gMainInterface->materialListModel->Regenerate();
-	gInterfaceReadyForSynchronization = true;
+	ui->widgetEffects->RegenerateLights();
+	ui->widgetDockFractal->RegeneratePrimitives();
+
 	gMainInterface->SynchronizeInterface(gPar, gParFractal, qInterface::write);
+	gInterfaceReadyForSynchronization = true;
+
 	gMainInterface->ComboMouseClickUpdate();
 	systemData.lastSettingsFile = gPar->Get<QString>("default_settings_path") + QDir::separator()
 																+ QFileInfo(filename).fileName();
@@ -347,7 +530,7 @@ void RenderWindow::slotMenuLoadPreset(QString filename)
 	gFlightAnimation->RefreshTable();
 	gKeyframeAnimation->RefreshTable();
 	showDescriptionPopup();
-	this->setWindowTitle(QString("Mandelbulber (") + systemData.lastSettingsFile + ")");
+	setWindowTitle(QString("Mandelbulber (") + systemData.lastSettingsFile + ")");
 }
 
 void RenderWindow::slotMenuRemovePreset(QString filename)
@@ -360,9 +543,9 @@ void RenderWindow::slotMenuRemovePreset(QString filename)
 void RenderWindow::slotPopulateCustomWindowStates(bool completeRefresh)
 {
 	WriteLog("cInterface::slotPopulateCustomWindowStates() started", 2);
-	QDir customWindowStateDir = QDir(systemData.GetCustomWindowStateFolder());
+	QDir customWindowStateDir = QDir(systemDirectories.GetCustomWindowStateFolder());
 	customWindowStateDir.setSorting(QDir::Time);
-	QStringList geometryFileExtension({"*.geometry"});
+	QStringList geometryFileExtension("*.geometry");
 	customWindowStateDir.setNameFilters(geometryFileExtension);
 	QStringList customWindowStateFiles =
 		customWindowStateDir.entryList(QDir::NoDotAndDotDot | QDir::Files);
@@ -371,9 +554,8 @@ void RenderWindow::slotPopulateCustomWindowStates(bool completeRefresh)
 
 	QList<QAction *> actions = ui->menuSaved_window_layouts->actions();
 	QStringList customWindowActions;
-	for (int i = 0; i < actions.size(); i++)
+	for (auto action : actions)
 	{
-		QAction *action = actions.at(i);
 		if (!action->objectName().startsWith("window_")) continue;
 		if (!customWindowStateFiles.contains(action->objectName()) || completeRefresh)
 		{
@@ -396,7 +578,7 @@ void RenderWindow::slotPopulateCustomWindowStates(bool completeRefresh)
 			continue;
 		}
 		QString filename =
-			systemData.GetCustomWindowStateFolder() + QDir::separator() + customWindowStateFile;
+			systemDirectories.GetCustomWindowStateFolder() + QDir::separator() + customWindowStateFile;
 
 		/*
 		QWidgetAction *action = new QWidgetAction(this);
@@ -405,11 +587,12 @@ void RenderWindow::slotPopulateCustomWindowStates(bool completeRefresh)
 		QToolButton *buttonRemove = new QToolButton;
 		QLabel *label = new QLabel;
 
-		buttonLoad->setStyleSheet("QLabel{ border: none; margin: 2px; padding: 1px; }");
+		buttonLoad->setStyleSheet("QLabel{ border: none; margin: 2px; padding: 1px;
+		}");
 		label->setText(QByteArray().fromBase64(QByteArray().append(customWindowStateFile)));
 		tooltipLayout->setContentsMargins(5, 0, 0, 0);
-		QIcon iconDelete = QIcon::fromTheme("list-remove", QIcon(":system/icons/list-remove.svg"));
-		buttonRemove->setIcon(iconDelete);
+		QIcon iconDelete = QIcon::fromTheme("list-remove",
+		QIcon(":system/icons/list-remove.svg")); buttonRemove->setIcon(iconDelete);
 		buttonRemove->setMaximumSize(QSize(15, 15));
 		tooltipLayout->addWidget(buttonRemove);
 		tooltipLayout->addWidget(label);
@@ -419,7 +602,7 @@ void RenderWindow::slotPopulateCustomWindowStates(bool completeRefresh)
 		action->setDefaultWidget(buttonLoad);*/
 
 		QAction *action = new QAction(this);
-		action->setText(QByteArray().fromBase64(QByteArray().append(customWindowStateFile)));
+		action->setText(QByteArray().fromBase64(QByteArray().append(customWindowStateFile.toUtf8())));
 		action->setObjectName("window_" + customWindowStateFile);
 
 		ui->menuSaved_window_layouts->addAction(action);
@@ -428,11 +611,17 @@ void RenderWindow::slotPopulateCustomWindowStates(bool completeRefresh)
 		QApplication::connect(action, SIGNAL(triggered()), mapCustomWindowLoad, SLOT(map()));
 		QApplication::processEvents();
 	}
+
+#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
 	QApplication::connect(mapCustomWindowLoad, SIGNAL(mapped(QString)), this,
 		SLOT(slotMenuLoadCustomWindowState(QString)));
+#else
+	QApplication::connect(mapCustomWindowLoad, SIGNAL(mappedString(QString)), this,
+		SLOT(slotMenuLoadCustomWindowState(QString)));
+#endif
 
 	/*QApplication::connect(mapCustomWindowRemove, SIGNAL(mapped(QString)), this,
-		SLOT(slotMenuRemoveCustomWindowState(QString)));*/
+	SLOT(slotMenuRemoveCustomWindowState(QString)));*/
 
 	WriteLog("cInterface::slotPopulateCustomWindowStates() finished", 2);
 }
@@ -444,15 +633,15 @@ void RenderWindow::slotCustomWindowStateAddToMenu()
 		tr("Enter a name for the new window settings"), QLineEdit::Normal, "", &ok);
 	if (!ok || text.isEmpty())
 	{
-		// qInfo() << "Cancelled window saving";
+		WriteLogCout("Cancelled window saving", 2);
 		return;
 	}
-	QString textEncoded = QByteArray().append(text).toBase64();
-	QString basePath = systemData.GetCustomWindowStateFolder() + QDir::separator();
+	QString textEncoded = QByteArray().append(text.toUtf8()).toBase64();
+	QString basePath = systemDirectories.GetCustomWindowStateFolder() + QDir::separator();
 	QString filename = basePath + textEncoded;
-	QString filenamGeometry = filename + ".geometry";
+	QString filenameGeometry = filename + ".geometry";
 	QString filenameState = filename + ".state";
-	QFile fileGeometry(filenamGeometry);
+	QFile fileGeometry(filenameGeometry);
 	QFile fileState(filenameState);
 	if (!fileGeometry.open(QIODevice::WriteOnly) || !fileState.open(QIODevice::WriteOnly))
 	{
@@ -476,28 +665,29 @@ void RenderWindow::slotMenuLoadCustomWindowState(QString filename)
 		qWarning() << "Could not open input files: " << filename << ".[geometry,state]";
 		return;
 	}
-	// this->restoreGeometry(fileGeometry.readAll());
-	this->restoreState(fileState.readAll());
+	// restoreGeometry(fileGeometry.readAll());
+	restoreState(fileState.readAll());
 }
 
 void RenderWindow::slotCustomWindowRemovePopup()
 {
-	QDir customWindowStateDir = QDir(systemData.GetCustomWindowStateFolder());
+	QDir customWindowStateDir = QDir(systemDirectories.GetCustomWindowStateFolder());
 	customWindowStateDir.setSorting(QDir::Time);
-	QStringList geometryFileExtension({"*.geometry"});
+	QStringList geometryFileExtension("*.geometry");
 	customWindowStateDir.setNameFilters(geometryFileExtension);
 	QStringList customWindowStateFiles =
 		customWindowStateDir.entryList(QDir::NoDotAndDotDot | QDir::Files);
 	QStringList itemsEscaped;
 	if (customWindowStateFiles.size() == 0)
 	{
-		// qInfo() << "Nothing to remove";
+		WriteLogCout("Nothing to remove", 2);
 		return;
 	}
 	for (int i = 0; i < customWindowStateFiles.size(); i++)
 	{
 		QString customWindowStateFile = customWindowStateFiles[i].replace(".geometry", "");
-		itemsEscaped.append(QByteArray().fromBase64(QByteArray().append(customWindowStateFile)));
+		itemsEscaped.append(
+			QByteArray().fromBase64(QByteArray().append(customWindowStateFile.toUtf8())));
 	}
 	bool ok;
 
@@ -505,12 +695,12 @@ void RenderWindow::slotCustomWindowRemovePopup()
 		tr("Select window setting to remove"), itemsEscaped, 0, false, &ok);
 	if (!ok || itemEscaped.isEmpty())
 	{
-		// qInfo() << "Cancelled window removing";
+		WriteLogCout("Cancelled window removing", 2);
 		return;
 	}
 	int index = itemsEscaped.indexOf(itemEscaped);
-	QString filename =
-		systemData.GetCustomWindowStateFolder() + QDir::separator() + customWindowStateFiles.at(index);
+	QString filename = systemDirectories.GetCustomWindowStateFolder() + QDir::separator()
+										 + customWindowStateFiles.at(index);
 	slotMenuRemoveCustomWindowState(filename.replace(".geometry", ""));
 }
 
@@ -521,6 +711,74 @@ void RenderWindow::slotMenuRemoveCustomWindowState(QString filename)
 	slotPopulateCustomWindowStates();
 }
 
+void RenderWindow::slotPopulateRecentSettings(bool completeRefresh)
+{
+	WriteLog("cInterface::slotPopulateRecentSettings() started", 2);
+	QFile recentFilesFile(systemDirectories.GetRecentFilesListFile());
+	if (!recentFilesFile.open(QFile::ReadOnly | QFile::Text))
+	{
+		// qDebug() << "cannot open recent file";
+		return;
+	}
+	QTextStream in(&recentFilesFile);
+	QString recentFilesFileContent = in.readAll();
+
+#if QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
+	QStringList recentFiles =
+		recentFilesFileContent.split(QRegExp("\n|\r\n|\r"), QString::KeepEmptyParts);
+#else
+	QStringList recentFiles =
+		recentFilesFileContent.split(QRegularExpression("\n|\r\n|\r"), Qt::KeepEmptyParts);
+#endif
+
+	QSignalMapper *mapRecentFileLoad = new QSignalMapper(this);
+	QList<QAction *> actions = ui->menuRecent_Settings_list->actions();
+	QStringList recentFileInActions;
+	for (auto action : actions)
+	{
+		if (!action->objectName().startsWith("recent_")) continue;
+		if (!recentFileInActions.contains(action->objectName()) || completeRefresh)
+		{
+			// preset has been removed
+			ui->menuRecent_Settings_list->removeAction(action);
+		}
+		else
+		{
+			recentFileInActions << action->objectName();
+		}
+	}
+
+	for (int i = 0; i < recentFiles.size(); i++)
+	{
+		QString settingsFile = recentFiles.at(i);
+		if (recentFileInActions.contains("recent_" + settingsFile))
+		{
+			// already present
+			continue;
+		}
+
+		QAction *action = new QAction(this);
+		action->setText(QFileInfo(settingsFile).baseName());
+		action->setObjectName("recent_" + settingsFile);
+
+		ui->menuRecent_Settings_list->addAction(action);
+
+		mapRecentFileLoad->setMapping(action, settingsFile);
+		QApplication::connect(action, SIGNAL(triggered()), mapRecentFileLoad, SLOT(map()));
+		QApplication::processEvents();
+	}
+
+#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
+	QApplication::connect(
+		mapRecentFileLoad, SIGNAL(mapped(QString)), this, SLOT(slotMenuLoadSettingsFromFile(QString)));
+#else
+	QApplication::connect(mapRecentFileLoad, SIGNAL(mappedString(QString)), this,
+		SLOT(slotMenuLoadSettingsFromFile(QString)));
+#endif
+
+	WriteLog("cInterface::slotPopulateRecentSettings() finished", 2);
+}
+
 void RenderWindow::slotQuit()
 {
 	gMainInterface->QuitApplicationDialog();
@@ -555,20 +813,20 @@ void RenderWindow::slotUpdateProgressAndStatus(const QString &text, const QStrin
 {
 	ui->statusbar->showMessage(text, 0);
 	MyProgressBar *progressBar = nullptr;
-	bool isQueue = this->sender() && this->sender()->objectName() == "Queue";
+	bool isQueue = false;
+
+	// FIXME: sender can be deleted by another thread (e.g. ImageFileSave exists
+	// very short time)
+	//  if (sender())
+	//  {
+	//     isQueue = sender()->objectName() == "Queue";
+	//  }
+
 	switch (progressType)
 	{
-		case cProgressText::progress_IMAGE:
-			if (isQueue)
-				progressBar = gMainInterface->progressBarQueueImage;
-			else
-				progressBar = gMainInterface->progressBar;
-			break;
+		case cProgressText::progress_IMAGE: progressBar = gMainInterface->progressBar; break;
 		case cProgressText::progress_ANIMATION:
-			if (isQueue)
-				progressBar = gMainInterface->progressBarQueueAnimation;
-			else
-				progressBar = gMainInterface->progressBarAnimation;
+			progressBar = gMainInterface->progressBarAnimation;
 			break;
 		case cProgressText::progress_QUEUE:
 			// nothing to be done, no progress bar for queue in GUI
@@ -657,10 +915,87 @@ void RenderWindow::slotAutoRefresh()
 void RenderWindow::slotMaterialSelected(int matIndex) const
 {
 	gMainInterface->MaterialSelected(matIndex);
-	ui->dockWidget_materialEditor->raise();
 }
 
 void RenderWindow::slotMaterialEdited()
 {
 	SynchronizeInterfaceWindow(gMainInterface->materialEditor, gPar, qInterface::write);
 }
+
+void RenderWindow::slotChangedComboGridType(int index)
+{
+	gMainInterface->renderedImage->SetGridType(RenderedImage::enumGridType(index));
+}
+
+void RenderWindow::ResetGlobalStopRequest()
+{
+	gMainInterface->ResetGlobalStopRequest();
+}
+
+bool RenderWindow::eventFilter(QObject *obj, QEvent *event)
+{
+	if (event->type() == QEvent::ToolTip && !gPar->Get<bool>("display_tooltips"))
+	{
+		{
+			return true;
+		}
+	}
+	else if (event->type() == QEvent::ShortcutOverride)
+	{
+		// buttonPressTimer->stop();
+		return QMainWindow::eventFilter(obj, event);
+	}
+	else
+	{
+		return QMainWindow::eventFilter(obj, event);
+	}
+}
+
+void RenderWindow::ToggleFullScreen()
+{
+	if (!ui->actionDetach_image_from_main_window->isChecked())
+	{
+		gMainInterface->DetachMainImageWidget();
+		gMainInterface->detachedWindow->showFullScreen();
+		ui->actionDetach_image_from_main_window->setChecked(true);
+	}
+	else
+	{
+		gMainInterface->AttachMainImageWidget();
+		ui->actionDetach_image_from_main_window->setChecked(false);
+	}
+}
+
+void RenderWindow::slotStartRender()
+{
+	// save setting to history folder
+	gMainInterface->SynchronizeInterface(gPar, gParFractal, qInterface::read);
+	cSettings parSettings(cSettings::formatCondensedText);
+	parSettings.CreateText(gPar, gParFractal, gAnimFrames, gKeyframes);
+	parSettings.SaveToFile(systemDirectories.GetHistoryFileName());
+
+	if (sender()->objectName() == "widgetDockNavigation")
+	{
+		bool correctionApplied = gMainInterface->CheckForMissingTextures();
+		if (correctionApplied)
+		{
+			gMainInterface->SynchronizeInterface(gPar, gParFractal, qInterface::write);
+		}
+	}
+
+	gMainInterface->StartRender();
+}
+
+void RenderWindow::slotCameraMovementModeChanged(int index)
+{
+	gMainInterface->renderedImage->SetCameraMovementMode(index);
+}
+
+void RenderWindow::slotShowMeasuremetDock()
+{
+	if (!ui->actionShow_measurement_dock->isChecked())
+	{
+		ui->actionShow_measurement_dock->setChecked(true);
+		slotUpdateDocksAndToolbarByAction();
+	}
+}
